# -*- coding: utf-8 -*-

# Copyright(C) 2015-2016 Julien Veyssier
# Copyright(C) 2016-2017 David Kremer
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.torrent import CapTorrent
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.compat import quote_plus
from weboob.tools.value import ValueBackendPassword, Value

from .browser import T411Browser


__all__ = ['T411Module']


class T411Module(Module, CapTorrent):
    NAME = 't411'
    MAINTAINER = u'David Kremer'
    EMAIL = 'courrier@david-kremer.fr'
    VERSION = '1.3'
    DESCRIPTION = 'T411 BitTorrent tracker'
    LICENSE = 'AGPLv3+'
    CONFIG = BackendConfig(Value('username', label='Username'), ValueBackendPassword('password', label='Password'))
    BROWSER = T411Browser

    def create_default_browser(self):
        return self.create_browser(self.config['username'].get(), self.config['password'].get())

    def get_torrent(self, torrent):
        return self.browser.get_torrent(torrent)

    def get_torrent_file(self, torrent):
        torrent = self.browser.get_torrent(torrent)
        if not torrent:
            return None

        resp = self.browser.open(torrent.url)
        return resp.content

    def iter_torrents(self, pattern):
        return self.browser.iter_torrents(quote_plus(pattern.encode('utf-8')))




# -*- coding: utf-8 -*-

# Copyright(C) 2013      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import PagesBrowser, URL

from .pages import ProgramPage, EventPage

__all__ = ['BiplanBrowser']


class BiplanBrowser(PagesBrowser):
    BASEURL = 'http://www.lebiplan.org'

    program_page = URL('/fr/biplan-prog-(?P<_category>.*).php', ProgramPage)
    event_page = URL('/(?P<_id>.*).html', EventPage)

    def list_events_concert(self, date_from, date_to=None, city=None, categories=None):
        return self.program_page.go(_category='concert').list_events(date_from=date_from,
                                                                     date_to=date_to,
                                                                     city=city,
                                                                     categories=categories,
                                                                     is_concert=True)

    def list_events_theatre(self, date_from, date_to=None, city=None, categories=None):
        return self.program_page.go(_category='theatre').list_events(date_from=date_from,
                                                                     date_to=date_to,
                                                                     city=city,
                                                                     categories=categories,
                                                                     is_concert=False)

    def get_event(self, _id, event=None):
        return self.event_page.go(_id=_id).get_event(obj=event)



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Clément Schreiner
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import logging
from copy import deepcopy
from PyQt5.QtCore import pyqtSlot as Slot
from PyQt5.QtWidgets import QMessageBox, QTableWidgetItem
from PyQt5.QtCore import Qt

from weboob.tools.application.base import MoreResultsAvailable
from weboob.tools.application.qt5 import QtMainWindow, QtDo
from weboob.tools.application.qt5.backendcfg import BackendCfg
from weboob.tools.application.qt5.models import BackendListModel
from weboob.capabilities.content import CapContent
from weboob.tools.misc import to_unicode

from .ui.main_window_ui import Ui_MainWindow


class MainWindow(QtMainWindow):
    def __init__(self, config, weboob, app, parent=None):
        super(MainWindow, self).__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.config = config
        self.weboob = weboob
        self.backend = None
        self.app = app

        self.ui.idEdit.returnPressed.connect(self.loadPage)
        self.ui.loadButton.clicked.connect(self.loadPage)
        self.ui.tabWidget.currentChanged.connect(self._currentTabChanged)
        self.ui.saveButton.clicked.connect(self.savePage)
        self.ui.actionBackends.triggered.connect(self.backendsConfig)
        self.ui.contentEdit.textChanged.connect(self._textChanged)
        self.ui.loadHistoryButton.clicked.connect(self.loadHistory)

        if hasattr(self.ui.descriptionEdit, "setPlaceholderText"):
            self.ui.descriptionEdit.setPlaceholderText("Edit summary")

        if self.weboob.count_backends() == 0:
            self.backendsConfig()
        else:
            self.loadBackends()

    @Slot()
    def backendsConfig(self):
        """ Opens backends configuration dialog when 'Backends' is clicked """
        bckndcfg = BackendCfg(self.weboob, (CapContent,), self)
        if bckndcfg.run():
            self.loadBackends()

    def loadBackends(self):
        """ Fills the backends comboBox with available backends """
        model = BackendListModel(self.weboob)
        model.addBackends(entry_all=False)
        self.ui.backendBox.setModel(model)

    @Slot()
    def _currentTabChanged(self):
        """ Loads history or preview when the corresponding tabs are shown """
        if self.ui.tabWidget.currentIndex() == 1:
            if self.backend is not None:
                self.loadPreview()
        elif self.ui.tabWidget.currentIndex() == 2:
            if self.backend is not None:
                self.loadHistory()

    @Slot()
    def _textChanged(self):
        """ The text in the content QPlainTextEdit has changed """
        if self.backend:
            self.ui.saveButton.setEnabled(True)
            self.ui.saveButton.setText('Save')

    @Slot()
    def loadPage(self):
        """ Loads a page's source into the 'content' QPlainTextEdit """
        _id = self.ui.idEdit.text()
        if not _id:
            return

        self.ui.loadButton.setEnabled(False)
        self.ui.loadButton.setText('Loading...')
        self.ui.contentEdit.setReadOnly(True)

        backend = self.ui.backendBox.currentText()

        def finished():
            self.process = None
            if self.backend:
                self.ui.contentEdit.setReadOnly(False)
                self.ui.loadButton.setEnabled(True)
                self.ui.loadButton.setText('Load')

        self.process = QtDo(self.weboob,
                            self._loadedPage,
                            self._errorLoadPage,
                            finished)
        self.process.do('get_content', _id, backends=(backend,))

    def _loadedPage(self, data):
        """ Callback for loadPage """
        if not data:
            self.content = None
            self.backend = None
            QMessageBox.critical(self, self.tr('Unable to open page'),
                                 'Unable to open page "%s" on %s: it does not exist.'
                                 % (self.ui.idEdit.text(),
                                    self.ui.backendBox.currentText()),
                                 QMessageBox.Ok)
            return

        self.content = data
        self.ui.contentEdit.setPlainText(self.content.content)
        self.setWindowTitle("QWebcontentedit - %s@%s" %(self.content.id,
                                                        self.content.backend))
        self.backend = self.weboob[self.content.backend]

    def _errorLoadPage(self, backend, error, backtrace):
        """ Error callback for loadPage """
        content = self.tr('Unable to load page:\n%s\n') % to_unicode(error)
        if logging.root.level <= logging.DEBUG:
            content += '\n%s\n' % to_unicode(backtrace)
        QMessageBox.critical(self, self.tr('Error while loading page'),
                             content, QMessageBox.Ok)
        self.ui.loadButton.setEnabled(True)
        self.ui.loadButton.setText("Load")

    @Slot()
    def savePage(self):
        """ Saves the current page to the remote site """
        if self.backend is None:
            return
        new_content = self.ui.contentEdit.toPlainText()
        minor = self.ui.minorBox.isChecked()
        if new_content != self.content.content:
            self.ui.saveButton.setEnabled(False)
            self.ui.saveButton.setText('Saving...')
            self.ui.contentEdit.setReadOnly(True)
            self.content.content = new_content
            message = self.ui.descriptionEdit.text()

            def finished():
                self.process = None
                self.ui.saveButton.setText('Saved')
                self.ui.descriptionEdit.clear()
                self.ui.contentEdit.setReadOnly(False)

            self.process = QtDo(self.weboob,
                                None,
                                self._errorSavePage,
                                finished)
            self.process.do('push_content',
                            self.content,
                            message,
                            minor=minor,
                            backends=self.backend)

    def _errorSavePage(self, backend, error, backtrace):
        """ """
        content = self.tr('Unable to save page:\n%s\n') % to_unicode(error)
        if logging.root.level <= logging.DEBUG:
            content += '\n%s\n' % to_unicode(backtrace)
        QMessageBox.critical(self, self.tr('Error while saving page'),
                             content, QMessageBox.Ok)
        self.ui.saveButton.setEnabled(True)
        self.ui.saveButton.setText("Save")

    def loadPreview(self):
        """ Loads the current page's preview into the preview QTextEdit """
        tmp_content = deepcopy(self.content)
        tmp_content.content = self.ui.contentEdit.toPlainText()
        self.ui.previewEdit.setHtml(self.backend.get_content_preview(tmp_content))

    @Slot()
    def loadHistory(self):
        """ Loads the page's log into the 'History' tab """
        if self.backend is None:
            return

        self.ui.loadHistoryButton.setEnabled(False)
        self.ui.loadHistoryButton.setText("Loading...")

        self.ui.historyTable.clear()
        self.ui.historyTable.setRowCount(0)

        self.ui.historyTable.setHorizontalHeaderLabels(["Revision",
                                                        "Time",
                                                        "Author",
                                                        "Summary"])
        self.ui.historyTable.setColumnWidth(3, 1000)

        def finished():
            self.process = None
            self.ui.loadHistoryButton.setEnabled(True)
            self.ui.loadHistoryButton.setText("Reload")


        self.process = QtDo(self.weboob,
                            self._gotRevision,
                            self._errorHistory,
                            finished)
        self.process.do(self.app._do_complete,
                        self.ui.nbRevBox.value(),
                        (),
                        'iter_revisions',
                        self.content.id,
                        backends=(self.backend,))

    def _gotRevision(self, revision):
        """ Callback for loadHistory's QtDo """
        # we set the flags to Qt.ItemIsEnabled so that the items
        # are not modifiable (they are modifiable by default)
        item_revision = QTableWidgetItem(revision.id)
        item_revision.setFlags(Qt.ItemIsEnabled)

        item_time = QTableWidgetItem(revision.timestamp.strftime('%Y-%m-%d %H:%M:%S'))
        item_time.setFlags(Qt.ItemIsEnabled)

        item_author = QTableWidgetItem(revision.author)
        item_author.setFlags(Qt.ItemIsEnabled)

        item_summary = QTableWidgetItem(revision.comment)
        item_summary.setFlags(Qt.ItemIsEnabled)

        row = self.ui.historyTable.currentRow() + 1
        self.ui.historyTable.insertRow(row)
        self.ui.historyTable.setItem(row, 0, item_revision)
        self.ui.historyTable.setItem(row, 1, item_time)
        self.ui.historyTable.setItem(row, 2, item_author)
        self.ui.historyTable.setItem(row, 3, item_summary)

        self.ui.historyTable.setCurrentCell(row, 0)

    def _errorHistory(self, backend, error, backtrace):
        """ Loading the history has failed """
        if isinstance(error, MoreResultsAvailable):
            return

        content = self.tr('Unable to load history:\n%s\n') % to_unicode(error)
        if logging.root.level <= logging.DEBUG:
            content += '\n%s\n' % to_unicode(backtrace)
        QMessageBox.critical(self, self.tr('Error while loading history'),
                             content, QMessageBox.Ok)

        self.ui.loadHistoryButton.setEnabled(True)
        self.ui.loadHistoryButton.setText("Reload")



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Clément Schreiner
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.capabilities.messages import CapMessages, Message, Thread
from weboob.tools.newsfeed import Newsfeed
from weboob.tools.value import Value


__all__ = ['NewsfeedModule']


class NewsfeedModule(Module, CapMessages):
    NAME = 'newsfeed'
    MAINTAINER = u'Clément Schreiner'
    EMAIL = "clemux@clemux.info"
    VERSION = '1.3'
    DESCRIPTION = "Loads RSS and Atom feeds from any website"
    LICENSE = "AGPLv3+"
    CONFIG = BackendConfig(Value('url', label="Atom/RSS feed's url", regexp='https?://.*'))
    STORAGE = {'seen': []}

    def iter_threads(self):
        for article in Newsfeed(self.config['url'].get()).iter_entries():
            yield self.get_thread(article.id, article)

    def get_thread(self, id, entry=None):
        if isinstance(id, Thread):
            thread = id
            id = thread.id
        else:
            thread = Thread(id)

        if entry is None:
            entry = Newsfeed(self.config['url'].get()).get_entry(id)
        if entry is None:
            return None

        flags = Message.IS_HTML
        if thread.id not in self.storage.get('seen', default=[]):
            flags |= Message.IS_UNREAD
        if len(entry.content) > 0:
            content = u"<p>Link %s</p> %s" % (entry.link, entry.content[0])
        else:
            content = entry.link

        thread.title = entry.title
        thread.root = Message(thread=thread,
                              id=0,
                              url=entry.link,
                              title=entry.title,
                              sender=entry.author,
                              receivers=None,
                              date=entry.datetime,
                              parent=None,
                              content=content,
                              children=[],
                              flags=flags)
        return thread

    def iter_unread_messages(self):
        for thread in self.iter_threads():
            for m in thread.iter_all_messages():
                if m.flags & m.IS_UNREAD:
                    yield m

    def set_message_read(self, message):
        self.storage.get('seen', default=[]).append(message.thread.id)
        self.storage.save()

    def fill_thread(self, thread, fields):
        return self.get_thread(thread)

    OBJECTS = {Thread: fill_thread}



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.capabilities.bank import Account
from weboob.capabilities.base import empty

from .pages import LoginPage, AccountsPage, TransactionsPage, AVAccountPage, AVHistoryPage, FormPage


__all__ = ['GanAssurances']


class GanAssurances(LoginBrowser):
    login = URL('/wps/portal/login.*',
                'https://authentification.(ganassurances|ganpatrimoine).fr/cas/login.*',
                '/wps/portal/inscription.*', LoginPage)
    accounts = URL('/wps/myportal/TableauDeBord', AccountsPage)
    transactions = URL('/wps/myportal/!ut.*', TransactionsPage)
    av_account_form = URL('/wps/myportal/assurancevie/rivage/!ut/p/a1/.*', FormPage)
    av_account = URL('https://secure-rivage.(ganassurances|ganpatrimoine).fr/contratVie.rivage.syntheseContratEparUc.gsi', AVAccountPage)
    av_history = URL('https://secure-rivage.(?P<website>.*).fr/contratVie.rivage.mesOperations.gsi', AVHistoryPage)

    def __init__(self, website, *args, **kwargs):
        self.BASEURL = 'https://%s' % website
        self.website = re.findall('espaceclient.(.*?).fr', self.BASEURL)[0]

        super(GanAssurances, self).__init__(*args, **kwargs)

    def do_login(self):
        """
        Attempt to log in.
        Note: this method does nothing if we are already logged in.
        """
        assert isinstance(self.username, basestring)
        assert isinstance(self.password, basestring)

        self.accounts.go()

        assert(self.login.is_here())

        self.page.login(self.username, self.password)

        if self.login.is_here() or '/login' in self.url:
            # sometimes ganassurances may be redirected to groupama.../login
            raise BrowserIncorrectPassword()

    # For life asssurance accounts, to get balance we use the link from the account.
    # And to get history (or other) we need to use the link again but the link works only once.
    # So we get balance only for iter_account to not use the new link each time.
    @need_login
    def get_accounts_list(self, balance=True):
        self.accounts.stay_or_go()
        a = self.page.get_list()
        for account in a:
            if account.type == Account.TYPE_LIFE_INSURANCE and balance:
                assert empty(account.balance)
                self.location(account._link)
                self.page.av_account_form()
                account.balance, account.currency = self.page.get_av_balance()
                self.location(self.BASEURL)
        return a

    def _get_history(self, account):
        accounts = self.get_accounts_list(balance=False)
        for a in accounts:
            if a.id == account.id:
                self.location(a._link)
                if a.type == Account.TYPE_LIFE_INSURANCE:
                    self.page.av_account_form()
                    self.av_history.go(website=self.website)
                    return self.page.get_av_history()
                assert self.transactions.is_here()
                return self.page.get_history(accid=account.id)
        return iter([])

    # Duplicate line in case of arbitration because the site has only one line for the 2 transactions (debit and credit on the same line)
    def get_history(self, account):
        for tr in self._get_history(account):
            yield tr
            if getattr(tr, '_arbitration', False):
                tr = tr.copy()
                tr.amount = -tr.amount
                yield tr

    def get_coming(self, account):
        if account.type == Account.TYPE_LIFE_INSURANCE:
            return iter([])
        accounts = self.get_accounts_list()
        for a in accounts:
            if a.id == account.id:
                self.location(a._link)
                assert self.transactions.is_here()

                link = self.page.get_coming_link()
                if link is not None:
                    self.location(self.page.get_coming_link())
                    assert self.transactions.is_here()

                    return self.page.get_history(accid=account.id)

        return iter([])

    def get_investment(self, account):
        if account.type != Account.TYPE_LIFE_INSURANCE:
            return iter([])
        accounts = self.get_accounts_list(balance=False)
        for a in accounts:
            if a.id == account.id:
                self.location(a._link)
                self.page.av_account_form()
                return self.page.get_av_investments()



"ArticlePage object for Taz newspaper"
# -*- coding: utf-8 -*-

# Copyright(C) 2012 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import AbstractPage
from weboob.browser.filters.html import CSS, CleanHTML


class ArticlePage(AbstractPage):
    "ArticlePage object for taz"
    _selector = CSS
    PARENT = 'genericnewspaper'
    PARENT_URL = 'generic_news_page'

    def on_loaded(self):
        self.main_div = self.doc.getroot()
        self.element_title_selector = "title"
        self.element_author_selector = 'a[rel="author"]>h4'

    def get_body(self):
        div = self.doc.getroot().find('.//div[@class="sectbody"]')
        self.try_drop_tree(div, "div.anchor")
        self.try_drop_tree(div, "script")
        self.clean_relativ_urls(div, "http://taz.de")
        return CleanHTML('.')(div)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Nicolas Duhamel
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


#~ from .pages.compose import ClosePage, ComposePage, ConfirmPage, SentPage
#~ from .pages.login import LoginPage

from .pages import LoginPage, ComposePage, ConfirmPage

from weboob.deprecated.browser import Browser, BrowserIncorrectPassword


__all__ = ['OrangeBrowser']


class OrangeBrowser(Browser):
    DOMAIN = 'orange.fr'
    PAGES = {
        'http://id.orange.fr/auth_user/bin/auth_user.cgi.*': LoginPage,
        'http://id.orange.fr/auth_user/bin/auth0user.cgi.*': LoginPage,
        'https://id.orange.fr/auth_user/bin/auth_user.cgi.*': LoginPage,
        'https://id.orange.fr/auth_user/bin/auth0user.cgi.*': LoginPage,
        'https://authweb.orange.fr/auth_user/bin/auth_user.cgi.*': LoginPage,
        'https://authweb.orange.fr/auth_user/bin/auth0user.cgi.*': LoginPage,
        'http://smsmms1.orange.fr/./Sms/sms_write.php.*'   : ComposePage,
        'http://smsmms1.orange.fr/./Sms/sms_write.php?command=send' : ConfirmPage,
        'https://smsmms1.orange.fr/./Sms/sms_write.php.*'   : ComposePage,
        'https://smsmms1.orange.fr/./Sms/sms_write.php?command=send' : ConfirmPage,
        }

    def get_nb_remaining_free_sms(self):
        self.location("http://smsmms1.orange.fr/M/Sms/sms_write.php")
        return self.page.get_nb_remaining_free_sms()

    def home(self):
        self.location("http://smsmms1.orange.fr/M/Sms/sms_write.php")

    def is_logged(self):
        self.location("http://smsmms1.orange.fr/M/Sms/sms_write.php", no_login=True)
        return not self.is_on_page(LoginPage)

    def login(self):
        if not self.is_on_page(LoginPage):
            self.location('https://authweb.orange.fr/auth_user/bin/auth_user.cgi?url=http://www.orange.fr', no_login=True)
        self.page.login(self.username, self.password)
        if not self.is_logged():
            raise BrowserIncorrectPassword()

    def post_message(self, message, sender):
        if not self.is_on_page(ComposePage):
            self.home()
        self.page.post_message(message, sender)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2014 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser.pages import HTMLPage, pagination
from weboob.browser.elements import ListElement, ItemElement, method
from weboob.browser.filters.standard import CleanText, Duration, Regexp
from weboob.browser.filters.html import Link, CSS
from weboob.capabilities.base import NotAvailable
from weboob.capabilities.image import Thumbnail
from weboob.capabilities.video import BaseVideo


class IndexPage(HTMLPage):
    @pagination
    @method
    class iter_videos(ListElement):
        item_xpath = '//span[@id="miniatura"]'

        next_page = Link(u'//a[text()="Next »"]')

        class item(ItemElement):
            klass = BaseVideo

            obj_id = CSS('a') & Link & Regexp(pattern=r'/videos/(.+)\.html')
            obj_title = CSS('span#title1') & CleanText
            obj_duration = CSS('span.thumbtime span') & CleanText & Duration | NotAvailable
            obj_nsfw = True

            def obj_thumbnail(self):
                thumbnail = Thumbnail(self.xpath('.//img')[0].attrib['data-original'])
                thumbnail.url = thumbnail.id.replace('http://', 'https://')
                return thumbnail



# -*- coding: utf-8 -*-
# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import os
from threading import RLock
from copy import copy

from weboob.capabilities.base import BaseObject, FieldNotFound, \
    Capability, NotLoaded, NotAvailable
from weboob.tools.misc import iter_fields
from weboob.tools.compat import basestring
from weboob.tools.log import getLogger
from weboob.tools.value import ValuesDict
from weboob.exceptions import ModuleInstallError


__all__ = ['BackendStorage', 'BackendConfig', 'Module']


class BackendStorage(object):
    """
    This is an abstract layer to store data in storages (:mod:`weboob.tools.storage`)
    easily.

    It is instancied automatically in constructor of :class:`Module`, in the
    :attr:`Module.storage` attribute.

    :param name: name of backend
    :param storage: storage object
    :type storage: :class:`weboob.tools.storage.IStorage`
    """

    def __init__(self, name, storage):
        self.name = name
        self.storage = storage

    def set(self, *args):
        """
        Set value in the storage.

        Example:

        >>> from weboob.tools.storage import StandardStorage
        >>> backend = BackendStorage('blah', StandardStorage('/tmp/cfg'))
        >>> backend.storage.set('config', 'nb_of_threads', 10)
        >>>

        :param args: the path where to store value
        """
        if self.storage:
            return self.storage.set('backends', self.name, *args)

    def delete(self, *args):
        """
        Delete a value from the storage.

        :param args: path to delete.
        """
        if self.storage:
            return self.storage.delete('backends', self.name, *args)

    def get(self, *args, **kwargs):
        """
        Get a value or a dict of values in storage.

        Example:

        >>> from weboob.tools.storage import StandardStorage
        >>> backend = BackendStorage('blah', StandardStorage('/tmp/cfg'))
        >>> backend.storage.get('config', 'nb_of_threads')
        10
        >>> backend.storage.get('config', 'unexistant', 'path', default='lol')
        'lol'
        >>> backend.storage.get('config')
        {'nb_of_threads': 10, 'other_things': 'blah'}

        :param args: path to get
        :param default: if specified, default value when path is not found
        """
        if self.storage:
            return self.storage.get('backends', self.name, *args, **kwargs)
        else:
            return kwargs.get('default', None)

    def load(self, default):
        """
        Load storage.

        It is made automatically when your backend is created, and use the
        ``STORAGE`` class attribute as default.

        :param default: this is the default tree if storage is empty
        :type default: :class:`dict`
        """
        if self.storage:
            return self.storage.load('backends', self.name, default)

    def save(self):
        """
        Save storage.
        """
        if self.storage:
            return self.storage.save('backends', self.name)


class BackendConfig(ValuesDict):
    """
    Configuration of a backend.

    This class is firstly instanced as a :class:`weboob.tools.value.ValuesDict`,
    containing some :class:`weboob.tools.value.Value` (and derivated) objects.

    Then, using the :func:`load` method will load configuration from file and
    create a copy of the :class:`BackendConfig` object with the loaded values.
    """
    modname = None
    instname = None
    weboob = None

    def load(self, weboob, modname, instname, config, nofail=False):
        """
        Load configuration from dict to create an instance.

        :param weboob: weboob object
        :type weboob: :class:`weboob.core.ouiboube.Weboob`
        :param modname: name of the module
        :type modname: :class:`str`
        :param instname: name of this backend
        :type instname: :class:`str`
        :param params: parameters to load
        :type params: :class:`dict`
        :param nofail: if true, this call can't fail
        :type nofail: :class:`bool`
        :rtype: :class:`BackendConfig`
        """
        cfg = BackendConfig()
        cfg.modname = modname
        cfg.instname = instname
        cfg.weboob = weboob
        for name, field in self.items():
            value = config.get(name, None)

            if value is None:
                if not nofail and field.required:
                    raise Module.ConfigError('Backend(%s): Configuration error: Missing parameter "%s" (%s)'
                                                  % (cfg.instname, name, field.description))
                value = field.default

            field = copy(field)
            try:
                field.load(cfg.instname, value, cfg.weboob.requests)
            except ValueError as v:
                if not nofail:
                    raise Module.ConfigError(
                        'Backend(%s): Configuration error for field "%s": %s' % (cfg.instname, name, v))

            cfg[name] = field
        return cfg

    def dump(self):
        """
        Dump config in a dictionary.

        :rtype: :class:`dict`
        """
        settings = {}
        for name, value in self.items():
            settings[name] = value.dump()
        return settings

    def save(self, edit=True, params=None):
        """
        Save backend config.

        :param edit: if true, it changes config of an existing backend
        :type edit: :class:`bool`
        :param params: if specified, params to merge with the ones of the current object
        :type params: :class:`dict`
        """
        assert self.modname is not None
        assert self.instname is not None
        assert self.weboob is not None

        dump = self.dump()
        if params is not None:
            dump.update(params)

        self.weboob.backends_config.add_backend(self.instname, self.modname, dump, edit)


class Module(object):
    """
    Base class for modules.

    You may derivate it, and also all capabilities you want to implement.

    :param weboob: weboob instance
    :type weboob: :class:`weboob.core.ouiboube.Weboob`
    :param name: name of backend
    :type name: :class:`str`
    :param config: configuration of backend
    :type config: :class:`dict`
    :param storage: storage object
    :type storage: :class:`weboob.tools.storage.IStorage`
    :param logger: logger
    :type logger: :class:`logging.Logger`
    """
    # Module name.
    NAME = None
    # Name of the maintainer of this module.
    MAINTAINER = u'<unspecified>'
    # Email address of the maintainer.
    EMAIL = '<unspecified>'
    # Version of module (for information only).
    VERSION = '<unspecified>'
    # Description
    DESCRIPTION = '<unspecified>'
    # License of this module.
    LICENSE = '<unspecified>'
    # Configuration required for backends.
    # Values must be weboob.tools.value.Value objects.
    CONFIG = BackendConfig()
    # Storage
    STORAGE = {}
    # Browser class
    BROWSER = None
    # URL to an optional icon.
    # If you want to create your own icon, create a 'favicon.ico' ico in
    # the module's directory, and keep the ICON value to None.
    ICON = None
    # Supported objects to fill
    # The key is the class and the value the method to call to fill
    # Method prototype: method(object, fields)
    # When the method is called, fields are only the one which are
    # NOT yet filled.
    OBJECTS = {}

    class ConfigError(Exception):
        """
        Raised when the config can't be loaded.
        """

    def __enter__(self):
        self.lock.acquire()

    def __exit__(self, t, v, tb):
        self.lock.release()

    def __repr__(self):
        return "<Backend %r>" % self.name

    def __init__(self, weboob, name, config=None, storage=None, logger=None, nofail=False):
        self.logger = getLogger(name, parent=logger)
        self.weboob = weboob
        self.name = name
        self.lock = RLock()
        if config is None:
            config = {}

        # Private fields (which start with '_')
        self._private_config = dict((key, value) for key, value in config.items() if key.startswith('_'))

        # Load configuration of backend.
        self.config = self.CONFIG.load(weboob, self.NAME, self.name, config, nofail)

        self.storage = BackendStorage(self.name, storage)
        self.storage.load(self.STORAGE)

    def deinit(self):
        """
        This abstract method is called when the backend is unloaded.
        """
        if self._browser is None:
            return

        if hasattr(self.browser, 'dump_state'):
            self.storage.set('browser_state', self.browser.dump_state())
            self.storage.save()
        if hasattr(self.browser, 'deinit'):
            self.browser.deinit()

    _browser = None

    @property
    def browser(self):
        """
        Attribute 'browser'. The browser is created at the first call
        of this attribute, to avoid useless pages access.

        Note that the :func:`create_default_browser` method is called to create it.
        """
        if self._browser is None:
            self._browser = self.create_default_browser()
        return self._browser

    def create_default_browser(self):
        """
        Method to overload to build the default browser in
        attribute 'browser'.
        """
        return self.create_browser()

    def create_browser(self, *args, **kwargs):
        """
        Build a browser from the BROWSER class attribute and the
        given arguments.

        :param klass: optional parameter to give another browser class to instanciate
        :type klass: :class:`weboob.browser.browsers.Browser`
        """

        klass = kwargs.pop('klass', self.BROWSER)

        if not klass:
            return None

        kwargs['proxy'] = self.get_proxy()
        kwargs['logger'] = self.logger

        if self.logger.settings['responses_dirname']:
            kwargs.setdefault('responses_dirname', os.path.join(self.logger.settings['responses_dirname'],
                                                                self._private_config.get('_debug_dir', self.name)))
        elif os.path.isabs(self._private_config.get('_debug_dir', '')):
            kwargs.setdefault('responses_dirname', self._private_config['_debug_dir'])
        if self._private_config.get('_highlight_el', ''):
            kwargs.setdefault('highlight_el', bool(int(self._private_config['_highlight_el'])))


        browser = klass(*args, **kwargs)

        if hasattr(browser, 'load_state'):
            browser.load_state(self.storage.get('browser_state', default={}))

        return browser

    def get_proxy(self):
        tmpproxy = None
        tmpproxys = None

        if '_proxy' in self._private_config:
            tmpproxy = self._private_config['_proxy']
        elif 'http_proxy' in os.environ:
            tmpproxy = os.environ['http_proxy']
        elif 'HTTP_PROXY' in os.environ:
            tmpproxy = os.environ['HTTP_PROXY']
        if '_proxy_ssl' in self._private_config:
            tmpproxys = self._private_config['_proxy_ssl']
        elif 'https_proxy' in os.environ:
            tmpproxys = os.environ['https_proxy']
        elif 'HTTPS_PROXY' in os.environ:
            tmpproxys = os.environ['HTTPS_PROXY']

        proxy = {}
        if tmpproxy is not None:
            proxy['http'] = tmpproxy
        if tmpproxys is not None:
            proxy['https'] = tmpproxys
        return proxy

    @classmethod
    def iter_caps(klass):
        """
        Iter capabilities implemented by this backend.

        :rtype: iter[:class:`weboob.capabilities.base.Capability`]
        """
        def iter_caps(cls):
            for base in cls.__bases__:
                if issubclass(base, Capability) and base != Capability:
                    yield base
                    for cap in iter_caps(base):
                        yield cap
        return iter_caps(klass)

    def has_caps(self, *caps):
        """
        Check if this backend implements at least one of these capabilities.
        """
        for c in caps:
            if (isinstance(c, basestring) and c in [cap.__name__ for cap in self.iter_caps()]) or \
               isinstance(self, c):
                return True
        return False

    def fillobj(self, obj, fields=None):
        """
        Fill an object with the wanted fields.

        :param fields: what fields to fill; if None, all fields are filled
        :type fields: :class:`list`
        """
        if obj is None:
            return obj

        def not_loaded_or_incomplete(v):
            return (v is NotLoaded or isinstance(v, BaseObject) and not v.__iscomplete__())

        def not_loaded(v):
            return v is NotLoaded

        def filter_missing_fields(obj, fields, check_cb):
            missing_fields = []
            if fields is None:
                # Select all fields
                if isinstance(obj, BaseObject):
                    fields = [item[0] for item in obj.iter_fields()]
                else:
                    fields = [item[0] for item in iter_fields(obj)]

            for field in fields:
                if not hasattr(obj, field):
                    raise FieldNotFound(obj, field)
                value = getattr(obj, field)

                missing = False
                if hasattr(value, '__iter__'):
                    for v in (value.values() if isinstance(value, dict) else value):
                        if check_cb(v):
                            missing = True
                            break
                elif check_cb(value):
                    missing = True

                if missing:
                    missing_fields.append(field)

            return missing_fields

        if isinstance(fields, basestring):
            fields = (fields,)

        missing_fields = filter_missing_fields(obj, fields, not_loaded_or_incomplete)

        if not missing_fields:
            return obj

        for key, value in self.OBJECTS.items():
            if isinstance(obj, key):
                self.logger.debug(u'Fill %r with fields: %s' % (obj, missing_fields))
                obj = value(self, obj, missing_fields) or obj
                missing_fields = filter_missing_fields(obj, fields, not_loaded)
                break

        # Object is not supported by backend. Do not notice it to avoid flooding user.
        # That's not so bad.
        for field in missing_fields:
            setattr(obj, field, NotAvailable)

        return obj


class AbstractModuleMissingParentError(Exception):
    pass


class AbstractModule(Module):
    """ Abstract module allow inheritance between modules.

    Sometimes, several websites are based on the same website engine. This module
    allow to simplify code with a fake inheritance: weboob will install (if needed) and
    load a PARENT module and build our AbstractModule on top of this class.

    PARENT is a mandatory attribute of any AbstractModule

    Note that you must pass a valid weboob instance as first argument of the constructor.
    """
    PARENT = None

    def __new__(cls, weboob, name, config=None, storage=None, logger=None, nofail=False):
        if cls.PARENT is None:
            raise AbstractModuleMissingParentError("PARENT is not defined for module %s" % cls.__name__)

        try:
            parent = weboob.load_or_install_module(cls.PARENT).klass
        except ModuleInstallError as err:
            raise ModuleInstallError('The module %s depends on %s module but %s\'s installation failed with: %s' % (name, cls.PARENT, cls.PARENT, err))

        cls.__bases__ = tuple([parent] + list(cls.iter_caps()))
        return object.__new__(cls)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value
from weboob.capabilities.job import CapJob, BaseJobAdvert

from .browser import ManpowerBrowser


__all__ = ['ManpowerModule']


class ManpowerModule(Module, CapJob):
    NAME = 'manpower'
    DESCRIPTION = u'manpower website'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = ManpowerBrowser

    type_contract_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        '': u'All',
        'cdi-interimaire/c11': u'Autre',
        'formation-en-alternance/c4': u'Alternance',
        'interim/c1': u'CDD',
        'cdd/c2': u'CDI',
        'cdi/c3': u'Mission en intérim',
    }.items())])

    activityDomain_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        '': u'All',
        'accueil-secretariat/s69': u'Accueil - Secrétariat',
        'achats-commerce-distribution/s66': u'Achats - Commerce - Distribution',
        'agro-alimentaire/s65': u'Agro-Alimentaire',
        'automobile/s2': u'Automobile',
        'banque-assurances-immobilier/s3': u'Banque - Assurances - Immobilier',
        'bijoux-horlogerie-lunetterie/s61': u'Bijoux - Horlogerie - Lunetterie',
        'bureau-d-etudes-methodes-qualite/s4': u'Bureau d\études- Méthodes - Qualité',
        'chimie-pharmacie-cosmetologie/s6': u'Chimie - Pharmacie - Cosmétologie',
        'communication/s73': u'Communication',
        'comptabilite-finance/s62': u'Comptabilité - Finance',
        'construction-travaux-publics/s9': u'Construction - Travaux publics',
        'electricite-electronique/s67': u'Electricité - Electronique',
        'environnement-developpement-durable/s80': u'Environnement - Développement Durable',
        'hotellerie-restauration-tourisme/s24': u'Hôtellerie- Restauration - Tourisme',
        'it-commercial-conseil-amoa/s75': u'IT - Commercial - Conseil - AMOA',
        'it-etude-et-developpement/s14': u'IT - Etude et Développement',
        'it-exploitation-systeme-sgbd/s76': u'IT - Exploitation - Système - SGBD',
        'it-reseau-telecom/s77': u'IT - Réseau - Telecom',
        'it-support-maintenance-help-desk/s78': u'IT - Support - Maintenance - Help Desk',
        'imprimerie/s12': u'Imprimerie',
        'industrie-aeronautique/s79': u'Industrie aéronautique',
        'logistique/s70': u'Logistique',
        'maintenance-entretien/s53': u'Maintenance - Entretien',
        'multimedia/s74': u'Multimédia',
        'metallurgie-fonderie/s49': u'Métallurgie- Fonderie',
        'naval/s47': u'Naval',
        'nucleaire-autres-energies/s54': u'Nucléaire - Autres Énergies',
        'papier-carton/s20': u'Papier - Carton',
        'plasturgie/s22': u'Plasturgie',
        'production-graphique/s72': u'Production Graphique',
        'production-industrielle-mecanique/s16': u'Production industrielle - Mécanique',
        'ressources-humaines-juridique/s63': u'Ressources humaines - Juridique',
        'sante/s25': u'Santé',
        'spectacle/s71': u'Spectacle',
        'surveillance-securite/s68': u'Surveillance - Sécurité',
        'textile-couture-cuir/s26': u'Textile - Couture - Cuir',
        'transport/s64': u'Transport',
        'transport-aerien/s52': u'Transport aérien',
        'teleservices-marketing-vente/s21': u'Téléservices - Marketing - Vente',
        'verre-porcelaine/s48': u'Verre - Porcelaine',
        'vin-agriculture-paysagisme/s60': u'Vin - Agriculture - Paysagisme',
    }.items())])

    places_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        '': u'All',
        'alsace/r01': u'Alsace',
        'alsace/bas-rhin/r01d67': u'Bas-Rhin',
        'alsace/haut-rhin/r01d68': u'Haut-Rhin',
        'aquitaine/r02': u'Aquitaine',
        'aquitaine/dordogne/r02d24': u'Dordogne',
        'aquitaine/gironde/r02d33': u'Gironde',
        'aquitaine/landes/r02d40': u'Landes',
        'aquitaine/lot-et-garonne/r02d47': u'Lot-et-Garonne',
        'aquitaine/pyrenees-atlantiques/r02d64': u'Pyrénées-Atlantiques',
        'auvergne/r03': u'Auvergne',
        'auvergne/allier/r03d3': u'Allier',
        'auvergne/cantal/r03d15': u'Cantal',
        'auvergne/haute-loire/r03d43': u'Haute-Loire',
        'auvergne/puy-de-dome/r03d63': u'Puy-de-Dôme',
        'basse-normandie/r04': u'Basse-Normandie',
        'basse-normandie/calvados/r04d14': u'Calvados',
        'basse-normandie/manche/r04d50': u'Manche',
        'basse-normandie/orne/r04d61': u'Orne',
        'bourgogne/r05': u'Bourgogne',
        'bourgogne/cote-d-or/r05d21': u'Côte-d\'Or',
        'bourgogne/nievre/r05d58': u'Nièvre',
        'bourgogne/saone-et-loire/r05d71': u'Saône-et-Loire',
        'bourgogne/yonne/r05d89': u'Yonne',
        'bretagne/r06': u'Bretagne',
        'bretagne/cotes-d-armor/r06d22': u'Côtes-d\'Armor',
        'bretagne/finistere/r06d29': u'Finistère',
        'bretagne/ille-et-vilaine/r06d35': u'Ille-et-Vilaine',
        'bretagne/morbihan/r06d56': u'Morbihan',
        'centre/r07': u'Centre',
        'centre/cher/r07d18': u'Cher',
        'centre/eure-et-loir/r07d28': u'Eure-et-Loir',
        'centre/indre/r07d36': u'Indre',
        'centre/indre-et-loire/r07d37': u'Indre-et-Loire',
        'centre/loir-et-cher/r07d41': u'Loir-et-Cher',
        'centre/loiret/r07d45': u'Loiret',
        'champagne-ardennes/r08': u'Champagne-Ardennes',
        'champagne-ardennes/ardennes/r08d8': u'Ardennes',
        'champagne-ardennes/aube/r08d10': u'Aube',
        'champagne-ardennes/haute-marne/r08d52': u'Haute-Marne',
        'champagne-ardennes/marne/r08d51': u'Marne',
        'dom-tom/r23': u'Dom Tom',
        'dom-tom/nouvelle-caledonie/r23d98': u'Nouvelle Calédonie',
        'franche-comte/r10': u'Franche-Comté',
        'franche-comte/doubs/r10d25': u'Doubs',
        'franche-comte/haute-saone/r10d70': u'Haute-Saône',
        'franche-comte/jura/r10d39': u'Jura',
        'franche-comte/territoire-de-belfort/r10d90': u'Territoire de Belfort',
        'haute-normandie/r11': u'Haute-Normandie',
        'haute-normandie/eure/r11d27': u'Eure',
        'haute-normandie/seine-maritime/r11d76': u'Seine-Maritime',
        'ile-de-france/r12': u'Île-de-France',
        'ile-de-france/essonne/r12d91': u'Essonne',
        'ile-de-france/hauts-de-seine/r12d92': u'Hauts-de-Seine',
        'ile-de-france/paris/r12d75': u'Paris',
        'ile-de-france/seine-st-denis/r12d93': u'Seine-St-Denis',
        'ile-de-france/seine-et-marne/r12d77': u'Seine-et-Marne',
        'ile-de-france/val-d-oise/r12d95': u'Val-d\'Oise',
        'ile-de-france/val-de-marne/r12d94': u'Val-de-Marne',
        'ile-de-france/yvelines/r12d78': u'Yvelines',
        'languedoc-roussillon/r13': u'Languedoc-Roussillon',
        'languedoc-roussillon/aude/r13d11': u'Aude',
        'languedoc-roussillon/gard/r13d30': u'Gard',
        'languedoc-roussillon/herault/r13d34': u'Hérault',
        'languedoc-roussillon/lozere/r13d48': u'Lozère',
        'languedoc-roussillon/pyrenees-orientales/r13d66': u'Pyrénées-Orientales',
        'limousin/r14': u'Limousin',
        'limousin/correze/r14d19': u'Corrèze',
        'limousin/creuse/r14d23': u'Creuse',
        'limousin/haute-vienne/r14d87': u'Haute-Vienne',
        'lorraine/r15': u'Lorraine',
        'lorraine/meurthe-et-moselle/r15d54': u'Meurthe-et-Moselle',
        'lorraine/meuse/r15d55': u'Meuse',
        'lorraine/moselle/r15d57': u'Moselle',
        'lorraine/vosges/r15d88': u'Vosges',
        'midi-pyrenees/r16': u'Midi-Pyrénées',
        'midi-pyrenees/ariege/r16d9': u'Ariège',
        'midi-pyrenees/aveyron/r16d12': u'Aveyron',
        'midi-pyrenees/gers/r16d32': u'Gers',
        'midi-pyrenees/haute-garonne/r16d31': u'Haute-Garonne',
        'midi-pyrenees/hautes-pyrenees/r16d65': u'Hautes-Pyrénées',
        'midi-pyrenees/lot/r16d46': u'Lot',
        'midi-pyrenees/tarn/r16d81': u'Tarn',
        'midi-pyrenees/tarn-et-garonne/r16d82': u'Tarn-et-Garonne',
        'nord-pas-de-calais/r17': u'Nord-Pas-de-Calais',
        'nord-pas-de-calais/nord/r17d59': u'Nord',
        'nord-pas-de-calais/pas-de-calais/r17d62': u'Pas-de-Calais',
        'pays-de-la-loire/r19': u'Pays de la Loire',
        'pays-de-la-loire/loire-atlantique/r19d44': u'Loire-Atlantique',
        'pays-de-la-loire/maine-et-loire/r19d49': u'Maine-et-Loire',
        'pays-de-la-loire/mayenne/r19d53': u'Mayenne',
        'pays-de-la-loire/sarthe/r19d72': u'Sarthe',
        'pays-de-la-loire/vendee/r19d85': u'Vendée',
        'picardie/r20': u'Picardie',
        'picardie/aisne/r20d2': u'Aisne',
        'picardie/oise/r20d60': u'Oise',
        'picardie/somme/r20d80': u'Somme',
        'poitou-charentes/r21': u'Poitou-Charentes',
        'poitou-charentes/charente/r21d16': u'Charente',
        'poitou-charentes/charente-maritime/r21d17': u'Charente-Maritime',
        'poitou-charentes/deux-sevres/r21d79': u'Deux-Sèvres',
        'poitou-charentes/vienne/r21d86': u'Vienne',
        'provence-alpes-cote-d-azur/r18': u'Provence-Alpes-Côte d\'Azur',
        'provence-alpes-cote-d-azur/alpes-maritimes/r18d6': u'Alpes-Maritimes',
        'provence-alpes-cote-d-azur/alpes-de-haute-provence/r18d4': u'Alpes-de-Haute-Provence',
        'provence-alpes-cote-d-azur/bouches-du-rhone/r18d13': u'Bouches-du-Rhône',
        'provence-alpes-cote-d-azur/hautes-alpes/r18d5': u'Hautes-Alpes',
        'provence-alpes-cote-d-azur/var/r18d83': u'Var',
        'provence-alpes-cote-d-azur/vaucluse/r18d84': u'Vaucluse',
        'rhone-alpes/r22': u'Rhône-Alpes',
        'rhone-alpes/ain/r22d1': u'Ain',
        'rhone-alpes/ardeche/r22d7': u'Ardèche',
        'rhone-alpes/drome/r22d26': u'Drôme',
        'rhone-alpes/haute-savoie/r22d74': u'Haute-Savoie',
        'rhone-alpes/isere/r22d38': u'Isère',
        'rhone-alpes/loire/r22d42': u'Loire',
        'rhone-alpes/rhone/r22d69': u'Rhône',
        'rhone-alpes/savoie/r22d73': u'Savoie',
    }.items())])

    CONFIG = BackendConfig(Value('job', label='Job name', masked=False, default=''),
                           Value('place', label=u'County', choices=places_choices, default=''),
                           Value('contract', labe=u'Contract type', choices=type_contract_choices, default=''),
                           Value('activity_domain', label=u'Activity Domain', choices=activityDomain_choices,
                                 default=''),
                           )

    def advanced_search_job(self):
        for advert in self.browser.advanced_search_job(job=self.config['job'].get(),
                                                       place=self.config['place'].get(),
                                                       contract=self.config['contract'].get(),
                                                       activity_domain=self.config['activity_domain'].get()):
            yield advert

    def get_job_advert(self, _id, advert=None):
        return self.browser.get_job_advert(_id, advert)

    def search_job(self, pattern=None):
        for advert in self.browser.search_job(pattern):
            yield advert

    def fill_obj(self, advert, fields):
        return self.get_job_advert(advert.id, advert)

    OBJECTS = {BaseJobAdvert: fill_obj}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Christophe Benz
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from prettytable import PrettyTable

from weboob.capabilities.base import empty
from weboob.tools.compat import range

from .iformatter import IFormatter


__all__ = ['TableFormatter', 'HTMLTableFormatter']


class TableFormatter(IFormatter):
    HTML = False

    def __init__(self):
        super(TableFormatter, self).__init__()
        self.queue = []
        self.keys = None
        self.header = None

    def flush(self):
        s = self.get_formatted_table()
        if s is not None:
            self.output(s)

    def get_formatted_table(self):
        if len(self.queue) == 0:
            return

        queue = [() for i in range(len(self.queue))]
        column_headers = []
        # Do not display columns when all values are NotLoaded or NotAvailable
        maxrow = 0
        for i in range(len(self.keys)):
            available = False
            for line in self.queue:
                if len(line)> i and not empty(line[i]):
                    maxrow += 1
                    available = True
                    break
            if available:
                column_headers.append(self.keys[i].capitalize().replace('_', ' '))
                for j in range(len(self.queue)):
                    if len(self.queue[j]) > i:
                        queue[j] += (self.queue[j][i],)

        s = ''
        if self.display_header and self.header:
            if self.HTML:
                s += '<p>%s</p>' % self.header
            else:
                s += self.header
            s += "\n"
        table = PrettyTable(list(column_headers))
        for column_header in column_headers:
            # API changed in python-prettytable. The try/except is a bad hack to support both versions
            # Note: two versions are not exactly the same...
            # (first one: header in center. Second one: left align for header too)
            try:
                table.set_field_align(column_header, 'l')
            except:
                table.align[column_header] = 'l'
        for line in queue:
            for _ in range(maxrow - len(line)):
                line += ('',)
            table.add_row(line)

        if self.HTML:
            s += table.get_html_string()
        else:
            s += table.get_string()

        self.queue = []

        return s

    def format_dict(self, item):
        if self.keys is None:
            self.keys = list(item.keys())
        self.queue.append(list(item.values()))

    def set_header(self, string):
        self.header = string


class HTMLTableFormatter(TableFormatter):
    HTML = True


def test():
    from .iformatter import formatter_test_output as fmt
    assert fmt(TableFormatter, {'foo': 'bar'}) == \
        '+-----+\n' \
        '| Foo |\n' \
        '+-----+\n' \
        '| bar |\n' \
        '+-----+\n'



# -*- coding: utf-8 -*-

# Copyright(C) 2014 Budget Insight
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from weboob.exceptions import BrowserIncorrectPassword
from weboob.browser import LoginBrowser, URL, need_login
from weboob.tools.date import new_date

from .pages import (
    LoginPage, ClientPage, OperationsPage, ChoicePage,
    CreditHome, CreditAccountPage, CreditHistory,
)

__all__ = ['OneyBrowser']


class OneyBrowser(LoginBrowser):
    BASEURL = 'https://www.oney.fr'

    login =       URL(r'/site/s/login/login.html', LoginPage)

    choice =      URL(r'/site/s/multimarque/choixsite.html', ChoicePage)
    choice_portal = URL(r'/site/s/login/loginidentifiant.html')

    client =      URL(r'/oney/client', ClientPage)
    operations =  URL(r'/oney/client', OperationsPage)
    card_page =   URL(r'/oney/client\?task=Synthese&process=SyntheseMultiCompte&indexSelectionne=(?P<acc_num>\d+)')

    credit_home = URL(r'/site/s/detailcompte/detailcompte.html', CreditHome)
    credit_info = URL(r'/site/s/detailcompte/ongletdetailcompte.html', CreditAccountPage)
    credit_hist = URL(r'/site/s/detailcompte/exportoperations.html', CreditHistory)

    has_oney = False
    has_other = False
    card_name = None

    def do_login(self):
        self.session.cookies.clear()

        self.login.go()

        self.page.login(self.username, self.password)

        if self.choice.is_here():
            self.has_other = self.has_oney = True
        elif self.credit_home.is_here():
            self.has_other = True
        elif self.client.is_here():
            self.has_oney = True
        else:
            raise BrowserIncorrectPassword()

    def go_site(self, site):
        if site == 'oney':
            if self.credit_home.is_here() or self.credit_info.is_here() or self.credit_hist.is_here():
                self.choice.go()
                assert self.choice.is_here()
            if self.choice.is_here():
                self.choice_portal.go(data={'selectedSite': 'ONEY_HISTO'})

        elif site == 'other':
            if self.client.is_here() or self.operations.is_here():
                self.do_login()
                assert self.choice.is_here()
            if self.choice.is_here():
                self.choice_portal.go(data={'selectedSite': 'ONEY'})

    @need_login
    def get_accounts_list(self):
        accounts = []

        if self.has_other:
            self.go_site('other')
            self.credit_home.stay_or_go()
            self.card_name = self.page.get_name()
            self.credit_info.go()
            accounts.append(self.page.get_account())

        if self.has_oney:
            self.go_site('oney')
            self.client.stay_or_go()
            accounts.extend(self.page.iter_accounts())

        return accounts

    def _build_hist_form(self):
        form = {}
        d = date.today()
        form['jourDebut'] = '1'
        form['moisDebut'] = '1'
        form['anneeDebut'] = '2016'
        form['jourFin'] = str(d.day)
        form['moisFin'] = str(d.month)
        form['anneeFin'] = str(d.year)
        form['typeOpe'] = 'deux'
        form['formatFichier'] = 'xls' # or pdf... great choice
        return form

    @need_login
    def iter_history(self, account):
        self.go_site(account._site)
        if account._site == 'oney':
            if account._num:
                self.card_page.go(acc_num=account._num)
            post = {'task': 'Synthese', 'process': 'SyntheseCompte', 'taskid': 'Releve'}
            self.operations.go(data=post)

            for tr in self.page.iter_transactions(seen=set()):
                yield tr

        elif account._site == 'other':
            self.credit_hist.go(params=self._build_hist_form())

            d = date.today().replace(day=1) # TODO is it the right date?
            for tr in self.page.iter_history():
                if new_date(tr.date) < d:
                    yield tr

    @need_login
    def iter_coming(self, account):
        self.go_site(account._site)
        if account._site == 'oney':
            if account._num:
                self.card_page.go(acc_num=account._num)
            post = {'task': 'OperationRecente', 'process': 'OperationRecente', 'taskid': 'OperationRecente'}
            self.operations.go(data=post)

            for tr in self.page.iter_transactions(seen=set()):
                yield tr

        elif account._site == 'other':
            self.credit_hist.go(params=self._build_hist_form())
            d = date.today().replace(day=1) # TODO is it the right date?
            for tr in self.page.iter_history():
                if new_date(tr.date) >= d:
                    yield tr




#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import print_function

import sys
import resources.lib.test.common_test as common_xbmc
import resources.lib.constants as constants

from resources.lib.actions import actions

print(sys.argv)
if len(sys.argv) < 2:
    actions[constants.DISPLAY_MENU]()._do()
else:
    params = common_xbmc.parse_params(sys.argv[1])
    #print params
    action = params.get("action")
    if (action):
        actions[action]()._do(params)
    else:
        common_xbmc.display_error(" ARGV Nothing done.. verify params " + repr(params))



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.elements import ItemElement, TableElement, method
from weboob.browser.filters.standard import CleanText, Date, Regexp, CleanDecimal, \
                                            TableCell, Field, Async, AsyncLoad, Eval
from weboob.browser.filters.html import Attr, Link
from weboob.capabilities.bank import Account, Investment, Transaction
from weboob.capabilities.base import NotAvailable, empty
from weboob.exceptions import BrowserUnavailable
from weboob.tools.compat import urljoin


def MyDecimal(*args, **kwargs):
    kwargs.update(replace_dots=True, default=NotAvailable)
    return CleanDecimal(*args, **kwargs)


class MaintenancePage(HTMLPage):
   def on_load(self):
        raise BrowserUnavailable(CleanText().filter(self.doc.xpath('//p')))


class LoginPage(HTMLPage):
    def login(self, login, password):
        form = self.get_form('//form[@id="loginForm"]')
        form['loginForm:name'] = login
        form['loginForm:password'] = password
        form['loginForm:login'] = "loginForm:login"
        form.submit()

    def get_error(self):
        return CleanText('//li[@class="erreurBox"]')(self.doc)


class AccountsPage(LoggedPage, HTMLPage):
    TYPES = {
        'Assurance Vie': Account.TYPE_LIFE_INSURANCE,
        'Capitalisation': Account.TYPE_MARKET,
        'Unknown': Account.TYPE_UNKNOWN,
    }

    @method
    class iter_accounts(TableElement):
        item_xpath = '//table[@role]/tbody/tr'
        head_xpath = '//table[@role]/thead/tr/th'

        col_label = u'Produit'
        col_id = u'Numéro de contrat'
        col_balance = u'Montant (€)'

        class item(ItemElement):
            klass = Account

            load_details = Field('url') & AsyncLoad

            obj_id = CleanText(TableCell('id'), replace=[(' ', '')])
            obj_label = CleanText(TableCell('label'))
            obj_balance = MyDecimal(TableCell('balance'))
            obj_valuation_diff = Async('details') & MyDecimal('//tr[1]/td[contains(text(), \
                                    "value du contrat")]/following-sibling::td')

            def obj_url(self):
                return urljoin(self.page.url, Link('.//a')(self))

            def obj_type(self):
                return self.page.TYPES[Async('details', CleanText('//td[contains(text(), \
                    "Option fiscale")]/following-sibling::td', default="Unknown"))(self)]


class TableInvestment(TableElement):
    col_label = u'Support'
    col_vdate = u'Date de valeur'
    col_unitvalue = u'Valeur de part'
    col_quantity = u'Nombre de parts'
    col_portfolio_share = u'%'


class ItemInvestment(ItemElement):
    klass = Investment

    obj_label = CleanText(TableCell('label'))
    obj_quantity = MyDecimal(TableCell('quantity', default=None))
    obj_unitvalue = MyDecimal(TableCell('unitvalue', default=None))
    obj_vdate = Date(CleanText(TableCell('vdate', default="")), dayfirst=True, default=NotAvailable)

    def obj_valuation(self):
        valuation = MyDecimal(TableCell('valuation', default=None))(self)
        h2 = CleanText('./ancestor::div[contains(@id, "Histo")][1]/preceding-sibling::h2[1]')(self)
        return -valuation if valuation and any(word in h2.lower() for word in self.page.DEBIT_WORDS) else valuation

    def obj_portfolio_share(self):
        ps = MyDecimal(TableCell('portfolio_share', default=None))(self)
        return Eval(lambda x: x / 100, ps)(self) if not empty(ps) else NotAvailable


class TableTransactionsInvestment(TableInvestment):
    item_xpath = './tbody/tr'
    head_xpath = './thead/tr/th'

    col_code = u'ISIN'
    col_valuation = [u'Montant brut', u'Montant net']

    class item(ItemInvestment):
        obj_code = Regexp(CleanText(TableCell('code')), pattern='([A-Z]{2}\d{10})', default=NotAvailable)


class ProfileTableInvestment(TableInvestment):
    # used only when portfolio is divided in multiple "profiles"
    head_xpath = '//thead[ends-with(@id, ":contratProfilTable_head")]/tr/th'


class DetailsPage(LoggedPage, HTMLPage):
    DEBIT_WORDS = [u'arrêté', 'rachat', 'frais', u'désinvestir']

    def get_investment_form(self):
        form = self.get_form('//form[contains(@id, "j_idt")]')
        form['ongletSituation:ongletContratTab_newTab'] = \
                Link().filter(self.doc.xpath('//a[contains(text(), "Prix de revient moyen")]'))[1:]
        form['javax.faces.source'] = "ongletSituation:ongletContratTab"
        form['javax.faces.behavior.event'] = "tabChange"
        return form

    @method
    class iter_investment(TableInvestment):
        item_xpath = '//div[contains(@id,"INVESTISSEMENT")]//div[ends-with(@id, ":tableDetailSituationCompte")]//table/tbody/tr[@data-ri]'
        head_xpath = '//div[contains(@id,"INVESTISSEMENT")]//div[ends-with(@id, ":tableDetailSituationCompte")]//table/thead/tr/th'

        col_valuation = re.compile('Contre')

        class item(ItemInvestment):
            obj_code = Regexp(CleanText('.//td[contains(text(), "Isin")]'), ':[\s]+([\w]+)', default=NotAvailable)

            def obj_unitprice(self):
                return MyDecimal('//div[contains(@id, "PRIX_REVIENT")]//a[contains(text(), \
                            "%s")]/ancestor::tr/td[5]' % Field('label')(self))(self)

            def obj_diff(self):
                return MyDecimal('//div[contains(@id, "PRIX_REVIENT")]//a[contains(text(), \
                            "%s")]/ancestor::tr/td[6]' % Field('label')(self))(self)

            def obj_portfolio_share(self):
                inv_share = ItemInvestment.obj_portfolio_share(self)
                if self.xpath('ancestor::tbody[ends-with(@id, "contratProfilTable_data")]'):
                    # investments are nested in profiles, row share is relative to profile share
                    profile_table_el = self.xpath('ancestor::tr/ancestor::table[position() = 1]')[0]
                    profile_table = ProfileTableInvestment(self.page, self, profile_table_el)
                    share_idx = profile_table.get_colnum('portfolio_share')
                    assert share_idx

                    path = 'ancestor::tr/preceding-sibling::tr[@data-ri][position() = 1][1]/td[%d]' % (share_idx + 1)

                    profile_share = MyDecimal(path)(self)
                    assert profile_share
                    #raise Exception('dtc')
                    profile_share = Eval(lambda x: x / 100, profile_share)(self)
                    return inv_share * profile_share
                else:
                    return inv_share

    def get_historytab_form(self):
        form = self.get_form('//form[contains(@id, "j_idt")]')
        idt = Attr(None, 'name').filter(self.doc.xpath('//input[contains(@name, "j_idt") \
                        and contains(@name, "activeIndex")]')).rsplit('_', 1)[0]
        form['%s_contentLoad' % idt] = "true"
        form['%s_newTab' % idt] = Link().filter(self.doc.xpath('//a[contains(@href, "HISTORIQUE")]'))[1:]
        form['%s_activeIndex' % idt] = "1"
        form['javax.faces.source'] = idt
        form['javax.faces.behavior.event'] = "tabChange"
        return form

    def get_historyallpages_form(self):
        onclick = self.doc.xpath('//a[contains(text(), "Tout")]/@onclick')
        if onclick:
            idt = re.search('{[^\w]+([\w\d:]+)', onclick[0]).group(1)
            form = self.get_form('//form[contains(@id, "j_idt")]')
            form[idt] = idt
            return form
        return False

    def get_historyexpandall_form(self):
        form = self.get_form('//form[contains(@id, "j_idt")]')
        form['javax.faces.source'] = "ongletHistoOperations:newoperations"
        form['javax.faces.behavior.event'] = "rowToggle"
        form['ongletHistoOperations:newoperations_rowExpansion'] = "true"
        for data in self.doc.xpath('//tr[@data-ri]/@data-ri'):
            form['ongletHistoOperations:newoperations_expandedRowIndex'] = data
            yield form

    @method
    class iter_history(TableElement):
        item_xpath = '//table/tbody[@id and not(contains(@id, "j_idt"))]/tr[@data-ri]'
        head_xpath = '//table/thead[@id and not(contains(@id, "j_idt"))]/tr/th'

        col_label = u'Type'
        col_status = u'Etat'
        col_brut = [u'Montant brut', u'Brut']
        col_net = [u'Montant net', u'Net']
        col_date = u'Date de réception'
        col_vdate = u'Date de valeur'

        class item(ItemElement):
            klass = Transaction

            obj_label = CleanText(TableCell('label'))
            obj_vdate = Date(CleanText(TableCell('vdate')), dayfirst=True)
            obj_type = Transaction.TYPE_BANK

            def obj_amount(self):
                amount = MyDecimal(TableCell('net') if not CleanText(TableCell('brut'))(self) else TableCell('brut'))(self)
                return -amount if amount and any(word in Field('label')(self).lower() for word in self.page.DEBIT_WORDS) else amount

            def obj_date(self):
                return Date(CleanText(TableCell('date')), dayfirst=True, default=Field('vdate')(self))(self)

            def condition(self):
                return u"Validé" in CleanText(TableCell('status'))(self) and u"Arrêté annuel" not in Field('label')(self)

            def obj_investments(self):
                investments = []
                for table in self.el.xpath('./following-sibling::tr[1]//span[contains(text(), "ISIN")]/ancestor::table[1]'):
                    investments.extend(TableTransactionsInvestment(self.page, el=table)())
                return investments



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from weboob.tools.test import BackendTest, skip_without_config


class TumblrTest(BackendTest):
    MODULE = 'tumblr'

    @skip_without_config('url')
    def test_tumblr(self):
        gall = self.backend.get_gallery('noname')
        assert gall
        assert gall.url

        for img, _ in zip(self.backend.iter_gallery_images(gall), range(10)):
            assert img.url
            self.backend.fillobj(img)
            assert img.data



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Kevin Pouget
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime

from weboob.browser import LoginBrowser, URL, need_login
from weboob.capabilities.base import find_object
from weboob.capabilities.bank import TransferInvalidDate
from weboob.exceptions import BrowserIncorrectPassword, BrowserUnavailable
from weboob.tools.date import new_date

from .pages import (
    LoginPage, CreditLoggedPage, AccountsPage, TransactionsPage,
    TransactionsJSONPage, ComingTransactionsPage, IbanPage,
    RecipientsPage, TransferPage, EmittersPage, TransferDatesPage,
    TransferValidatePage, TransferPostPage, TransferFinishPage,
)


__all__ = ['CreditCooperatif']


class CreditCooperatif(LoginBrowser):
    BASEURL = "https://www.credit-cooperatif.coop"

    loginpage = URL('/portail//particuliers/login.do', LoginPage)
    loggedpage = URL('/portail/particuliers/authentification.do', CreditLoggedPage)
    accountspage = URL('/portail/particuliers/mescomptes/synthese.do', AccountsPage)
    transactionpage = URL('/portail/particuliers/mescomptes/relevedesoperations.do', TransactionsPage)
    transactjsonpage = URL('/portail/particuliers/mescomptes/relevedesoperationsjson.do', TransactionsJSONPage)
    pre_comingpage = URL('/portail/particuliers/mescomptes/synthese/operationsencourslien.do', ComingTransactionsPage)
    comingpage = URL('/portail/particuliers/mescomptes/operationsavenir/avenir.do', ComingTransactionsPage)
    iban = URL('/portail/particuliers/mesoperations/ribiban/telechargementribajax.do\?accountExternalNumber=(?P<account_id>.*)', IbanPage)

    transfer_start = URL(r'/portail/particuliers/mesoperations/virement/creer.do', TransferPage)
    emitters = URL(r'/portail/particuliers/mesoperations/virement/singledebitaccountsajax.do', EmittersPage)
    recipients = URL(r'/portail/particuliers/mesoperations/virement/singlecreditaccountsajax.do', RecipientsPage)

    transfer_date = URL(r'/portail/particuliers/mesoperations/virement/formvirponctajax.do', TransferDatesPage)
    transfer_validate = URL(r'/portail/particuliers/mesoperations/virement/creer/validerajax.do', TransferValidatePage)
    transfer_post = URL(r'/portail/particuliers/mesoperations/virement/creer/challenge.do', TransferPostPage)
    transfer_finish = URL(r'/portail/particuliers/mesoperations/virement/creer/executerajax.do', TransferFinishPage)

    def do_login(self):
        self.loginpage.stay_or_go()
        self.page.login(self.username, self.password)

        if self.loggedpage.is_here():
            error = self.page.get_error()
            if error is None:
                return
        else:
            raise BrowserUnavailable("not on the login page")

        raise BrowserIncorrectPassword(error)

    @need_login
    def get_accounts_list(self):
        self.accountspage.go()

        return self.page.get_list()

    @need_login
    def get_history(self, account):
        data = {'accountExternalNumber': account.id}
        self.transactionpage.go(data=data)

        data = {'iDisplayLength':  400,
                'iDisplayStart':   0,
                'iSortCol_0':      0,
                'iSortingCols':    1,
                'sColumns':        '',
                'sEcho':           1,
                'sSortDir_0':      'asc',
                }
        self.transactjsonpage.go(data=data)

        return self.page.get_transactions()

    @need_login
    def get_coming(self, account):
        data = {'accountExternalNumber': account.id}
        self.pre_comingpage.go(data=data)
        assert self.comingpage.is_here()
        # this page is "operations du jour" and may not have any date yet
        # so don't take transactions here, but it sets the "context"

        self.comingpage.go()
        assert self.comingpage.is_here()
        for tr in self.page.get_transactions():
            yield tr

    @need_login
    def iter_recipients(self, account_id):
        self.transfer_start.go()
        self.emitters.go(data={
            'typevirradio': 'ponct',
        })
        if find_object(self.page.iter_emitters(), id=account_id) is None:
            return []

        self.recipients.go(data={
            'typevirradio': 'ponct',
            'nCompteDeb': account_id,
        })
        return self.page.iter_recipients()

    @need_login
    def init_transfer(self, transfer):
        date = new_date(transfer.exec_date or datetime.now())

        self.transfer_start.go()
        transfer_page = self.page

        self.emitters.go(data={
            'typevirradio': 'ponct',
        })
        self.recipients.go(data={
            'typevirradio': 'ponct',
            'nCompteDeb': transfer.account_id,
        })
        all_recipients = list(self.page.iter_recipients())

        self.transfer_date.go(data={
            'nCompteCred': transfer.recipient_id,
        })

        for page_date in self.page.iter_dates():
            if page_date >= date:
                date = page_date
                break
        else:
            raise TransferInvalidDate('The bank proposes no date greater or equal to the desired date')

        form = transfer_page.prepare_form(transfer=transfer, date=page_date)
        form.url = self.transfer_validate.build()
        form.submit()
        assert self.transfer_validate.is_here()

        form = transfer_page.prepare_form(transfer=transfer, date=page_date)
        form.url = self.transfer_post.build()
        form.submit()
        assert self.transfer_post.is_here()

        ret = self.page.get_transfer()
        if ret.recipient_iban:
            assert not ret.recipient_id # it's nowhere on the page
            recipient = find_object(all_recipients, iban=ret.recipient_iban)
            assert recipient
            ret.recipient_id = recipient.id
        return ret

    @need_login
    def execute_transfer(self, transfer):
        assert self.transfer_post.is_here()
        self.transfer_finish.go(method='POST', data='')
        assert self.transfer_finish.is_here()
        self.accountspage.go()
        return transfer



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Baptiste Delpey
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from io import BytesIO
import hashlib
from decimal import Decimal
from datetime import datetime

from weboob.capabilities.bank import Account
from weboob.exceptions import BrowserIncorrectPassword
from weboob.tools.capabilities.bank.transactions import FrenchTransaction
from weboob.browser.pages import HTMLPage, JsonPage, LoggedPage
from weboob.tools.captcha.virtkeyboard import MappedVirtKeyboard, VirtKeyboardError


class BNPVirtKeyboard(MappedVirtKeyboard):
    symbols = {'0': 'ff069462836e30a39c911034048f5bb3',
               '1': '7969f04e4e82eaefa2ce7a9a23c26178',
               '2': '1e6020f97ca1c3ce3da4f39ded15d67d',
               '3': 'f84284b40aea93c24814e23e14e76cc8',
               '4': '88bab262d4b344c0ef8f06ddd01adbcf',
               '5': '0a270764fc5d8334bcb55053432b26cb',
               '6': 'e6a4444a6c752cd3e655f2883e530080',
               '7': '933d4ca5df6b2b3df2dea00a21a3fed6',
               '8': ['f28b918777d21a5fde2bffb9899e2138', 'a97e6e27159084d50f8ef00548b70252'],
               '9': 'be751b77af0d998ab4c2cfd38455b2a6',
               }

    color=(0,0,0)

    def __init__(self, basepage):
        img = basepage.doc.xpath('//img[@id="gridpass_img"]')[0]
        imgdata = basepage.browser.open(img.attrib['src']).content
        MappedVirtKeyboard.__init__(self, BytesIO(imgdata), basepage.doc, img, self.color, convert='RGB')
        self.check_symbols(self.symbols, basepage.browser.responses_dirname)

    def get_symbol_code(self, md5sum):
        code = MappedVirtKeyboard.get_symbol_code(self, md5sum)
        code = code.split("'")[3]
        assert code.isdigit()
        return code

    def check_color(self, pixel):
        for p in pixel:
            if p >= 200:
                return False
        return True

    def checksum(self, coords):
        """Copy of parent checksum(), but cropping (removes empty lines)"""
        x1, y1, x2, y2 = coords
        s = ''
        for y in range(y1, min(y2 + 1, self.height)):
            for x in range(x1, min(x2 + 1, self.width)):
                if self.check_color(self.pixar[x, y]):
                    s += " "
                else:
                    s += "O"
            s += "\n"
        s = '\n'.join([l for l in s.splitlines() if l.strip()])
        return hashlib.md5(s).hexdigest()


class LoginPage(HTMLPage):
    def login(self, login, password):
        try:
            vk = BNPVirtKeyboard(self)
        except VirtKeyboardError as err:
            self.logger.error("Error: %s" % err)
            return False

        form = self.get_form(name='loginPwdForm')
        form['txtAuthentMode'] = 'PASSWORD'
        form['txtPwdUserId'] = login
        form['gridpass_hidden_input'] = vk.get_string_code(password)
        form.submit()

    def on_load(self):
        if self.doc.xpath('//p[contains(text(), "Your identification is wrong.")]'):
            raise BrowserIncorrectPassword("Your identification is wrong.")


class AccountsPage(LoggedPage, JsonPage):
    FAMILY_TO_TYPE = {
        u"Compte chèque": Account.TYPE_CHECKING,
    }

    def iter_accounts(self):
        for f in self.path('tableauSoldes.listeGroupes'):
            for g in f:
                for a in g.get('listeComptes'):
                    yield Account.from_dict({
                        'id': a.get('numeroCompte'),
                        'iban': a.get('numeroCompte'),
                        'type': self.FAMILY_TO_TYPE.get(a.get('libelleType')) or Account.TYPE_UNKNOWN,
                        'label': '%s %s' % (a.get('libelleType'), a.get('libelleTitulaire')),
                        'currency': a.get('deviseTenue'),
                        'balance': Decimal(a.get('soldeComptable')) / 100,
                        'coming': Decimal(a.get('soldePrevisionnel')) / 100,
                    })


class HistoryPage(LoggedPage, JsonPage):
    CODE_TO_TYPE = {
        "AUTOP": FrenchTransaction.TYPE_UNKNOWN, # Autres opérations,
        "BOURS": FrenchTransaction.TYPE_BANK, # Bourse / Titres,
        "CARTE": FrenchTransaction.TYPE_CARD, # Cartes,
        "CHEQU": FrenchTransaction.TYPE_CHECK, # Chèques,
        "CREDD": FrenchTransaction.TYPE_UNKNOWN, # Crédits documentaires,
        "CREDI": FrenchTransaction.TYPE_UNKNOWN, # Crédits,
        "EFFET": FrenchTransaction.TYPE_UNKNOWN, # Effets,
        "ESPEC": FrenchTransaction.TYPE_UNKNOWN, # Espèces,
        "FACCB": FrenchTransaction.TYPE_UNKNOWN, # Factures / Retraits cartes,
        "ICHEQ": FrenchTransaction.TYPE_UNKNOWN, # Impayés chèques,
        "IEFFE": FrenchTransaction.TYPE_UNKNOWN, # Impayés et incidents effets,
        "IMPAY": FrenchTransaction.TYPE_UNKNOWN, # Impayés et rejets,
        "IPRLV": FrenchTransaction.TYPE_UNKNOWN, # Impayés prélèvements, TIP et télérèglements,
        "PRLVT": FrenchTransaction.TYPE_UNKNOWN, # Prélèvements, TIP et télérèglements,
        "REMCB": FrenchTransaction.TYPE_UNKNOWN, # Remises cartes,
        "RJVIR": FrenchTransaction.TYPE_ORDER, # Rejets de virements,
        "VIREM": FrenchTransaction.TYPE_ORDER, # Virements,
        "VIRIT": FrenchTransaction.TYPE_ORDER, # Virements internationaux,
        "VIRSP": FrenchTransaction.TYPE_ORDER, # Virements européens,
        "VIRTR": FrenchTransaction.TYPE_ORDER, # Virements de trésorerie,
        "VIRXX": FrenchTransaction.TYPE_ORDER, # Autres virements
    }

    def one(self, path, context=None):
        try:
            return list(self.path(path, context))[0]
        except IndexError:
            return None

    def iter_history(self):
        for op in self.get('mouvementsBDDF'):
            codeFamille = self.one('nature.codefamille', op)
            tr = FrenchTransaction.from_dict({
                'id': op.get('id'),
                'type': self.CODE_TO_TYPE.get(codeFamille) or FrenchTransaction.TYPE_UNKNOWN,
                'category': self.one('nature.libelle', op),
                'raw': ' '.join(op.get('libelle').split()) or op.get('nature')['libelle'],
                'date': datetime.fromtimestamp(op.get('dateOperation') / 1000),
                'vdate': datetime.fromtimestamp(op.get('dateValeur') / 1000),
                'amount': Decimal(self.one('montant.montant', op)) / 100,
            })
            yield tr



# -*- coding: utf-8 -*-

# Copyright(C) 2014  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re
from decimal import Decimal

from weboob.browser.elements import ListElement, ItemElement, method
from weboob.browser.filters.standard import CleanText, CleanDecimal, Date
from weboob.browser.filters.html import Link
from weboob.browser.pages import LoggedPage, pagination
from weboob.capabilities.bank import Account
from weboob.tools.compat import urljoin, unicode

from .accounthistory import Transaction
from .base import MyHTMLPage


class RedirectPage(LoggedPage, MyHTMLPage):
    def check_for_perso(self):
        return self.doc.xpath(u'//p[contains(text(), "L\'identifiant utilisé est celui d\'un compte de Particuliers")]')


class ProAccountsList(LoggedPage, MyHTMLPage):
    ACCOUNT_TYPES = {u'comptes titres': Account.TYPE_MARKET,
                     u'comptes Ã©pargne':    Account.TYPE_SAVINGS,
                     # wtf? ^
                     u'comptes épargne':     Account.TYPE_SAVINGS,
                     u'comptes courants':    Account.TYPE_CHECKING,
                    }
    def get_accounts_list(self):
        for table in self.doc.xpath('//div[@class="comptestabl"]/table'):
            try:
                account_type = self.ACCOUNT_TYPES[table.get('summary').lower()]
                if not account_type:
                    account_type = self.ACCOUNT_TYPES[table.xpath('./caption/text()')[0].strip().lower()]
            except (IndexError,KeyError):
                account_type = Account.TYPE_UNKNOWN
            for tr in table.xpath('./tbody/tr'):
                cols = tr.findall('td')

                link = cols[0].find('a')
                if link is None:
                    continue

                a = Account()
                a.type = account_type
                a.id = unicode(re.search('([A-Z\d]{4}[A-Z\d\*]{3}[A-Z\d]{4})', link.attrib['title']).group(1))
                a.label = unicode(link.attrib['title'].replace('%s ' % a.id, ''))
                tmp_balance = CleanText(None).filter(cols[1])
                a.currency = a.get_currency(tmp_balance)
                if not a.currency:
                    a.currency = u'EUR'
                a.balance = Decimal(Transaction.clean_amount(tmp_balance))
                a._has_cards = False
                a.url = urljoin(self.url, link.attrib['href'])
                yield a


class ProAccountHistory(LoggedPage, MyHTMLPage):
    @pagination
    @method
    class iter_history(ListElement):
        item_xpath = u'//div[@id="tabReleve"]//tbody/tr'
        next_page = Link('//div[@class="pagination"]//li[@class="pagin-on-right"]/a')

        class item(ItemElement):
            klass = Transaction

            obj_date = Date(CleanText('.//td[@headers="date"]'), dayfirst=True)
            obj_raw = Transaction.Raw('.//td[@headers="libelle"]')
            obj_amount = CleanDecimal('.//td[@headers="debit" or @headers="credit"]', replace_dots=True)


class DownloadRib(LoggedPage, MyHTMLPage):
    def get_rib_value(self, acc_id):
        opt = self.doc.xpath('//div[@class="rechform"]//option')
        for o in opt:
            if acc_id in o.text:
                return o.xpath('./@value')[0]
        return None


class RibPage(LoggedPage, MyHTMLPage):
    def get_iban(self):
        if self.doc.xpath('//div[@class="blocbleu"][2]//table[@class="datalist"]'):
            return CleanText()\
                .filter(self.doc.xpath('//div[@class="blocbleu"][2]//table[@class="datalist"]')[0])\
                .replace(' ', '').strip()
        return None




# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Cedric Defortis
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import JsonPage
from weboob.browser.elements import ItemElement, DictElement, method
from weboob.capabilities.weather import Forecast, Current, City, Temperature
from weboob.browser.filters.json import Dict
from weboob.browser.filters.standard import CleanText, CleanDecimal, Format, Date, Env


class YahooPage(JsonPage):
    @method
    class iter_cities(DictElement):
        item_xpath = 'query/results/place'

        class item(ItemElement):
            klass = City

            obj_id = Dict('woeid')
            obj_name = Format(u'%s, %s, %s', Dict('name'), Dict('admin1/content'), Dict('country/content'))

    @method
    class get_current(ItemElement):
        klass = Current

        def parse(self, el):
            self.env['pct'] = u'%'

        obj_id = Date(Dict('query/results/channel/item/condition/date'))
        obj_date = Date(Dict('query/results/channel/item/condition/date'))
        obj_text = Format('%s - wind: %s%s - humidity:%s%s',
                          Dict('query/results/channel/item/condition/text'),
                          Dict('query/results/channel/wind/speed'),
                          Dict('query/results/channel/units/speed'),
                          Dict('query/results/channel/atmosphere/humidity'),
                          Env('pct'))

        def obj_temp(self):
            temp = CleanDecimal(Dict('query/results/channel/item/condition/temp'))(self)
            unit = CleanText(Dict('query/results/channel/units/temperature'))(self)
            return Temperature(float(temp), unit)

    @method
    class iter_forecast(DictElement):
        item_xpath = 'query/results/channel/item/forecast'

        def parse(self, el):
            self.env['unit'] = Dict('query/results/channel/units/temperature')(el)

        class item(ItemElement):
            klass = Forecast

            obj_id = Dict('date')
            obj_date = Date(Dict('date'))
            obj_text = Dict('text')

            def obj_low(self):
                temp = CleanDecimal(Dict('low'))(self)
                return Temperature(float(temp), Env('unit')(self))

            def obj_high(self):
                temp = CleanDecimal(Dict('high'))(self)
                return Temperature(float(temp), Env('unit')(self))



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.tools.date import LinearDateGuesser

from .pages import LoginPage, LoginErrorPage, GroupamaesPage


__all__ = ['GroupamaesBrowser']


class GroupamaesBrowser(LoginBrowser):
    BASEURL = 'https://www.gestion-epargne-salariale.fr'

    login = URL('/groupama-es/fr/index.html', LoginPage)
    login_error = URL('/groupama-es/fr/identification/default.cgi', LoginErrorPage)
    groupamaes_page = URL('/groupama-es/fr/espace/devbavoirs.aspx\?mode=net&menu=cpte(?P<page>.*)', GroupamaesPage)

    def do_login(self):
        self.login.stay_or_go()

        self.page.login(self.username, self.password)

        if not self.page.logged or self.login_error.is_here():
            raise BrowserIncorrectPassword()

    @need_login
    def get_accounts_list(self):
        return self.groupamaes_page.stay_or_go(page='&page=situglob').iter_accounts()

    @need_login
    def get_history(self):
        transactions = list(self.groupamaes_page.go(page='&_pid=MenuOperations&_fid=GoOperationsTraitees').get_history(date_guesser=LinearDateGuesser()))
        transactions.sort(key=lambda tr: tr.rdate, reverse=True)
        return transactions

    @need_login
    def get_coming(self):
        transactions = list(self.groupamaes_page.go(page='&_pid=OperationsTraitees&_fid=GoWaitingOperations').get_history(date_guesser=LinearDateGuesser(), coming=True))
        transactions.sort(key=lambda tr: tr.rdate, reverse=True)
        return transactions

    @need_login
    def iter_investment(self):
        return self.groupamaes_page.go(page='&_fid=GoPositionsParFond&_pid=SituationGlobale').iter_investment()



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon, Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest
from weboob.capabilities.bank import Account
from datetime import timedelta
import random


class INGTest(BackendTest):
    MODULE = 'ing'

    def test_accounts(self):
        l = list(self.backend.iter_accounts())
        for account in l:
            # Test if get_account works
            _id = self.backend.get_account(account.id)
            self.assertTrue(_id.id == account.id)
            # Methods can use Account objects or id. Try one of them
            id_or_account = random.choice([account, account.id])
            if account.type == Account.TYPE_CHECKING or account.type == Account.TYPE_SAVINGS:
                history = list(self.backend.iter_history(id_or_account))
                self.assertTrue(len(history) > 0)
                date = history.pop(0).date
                for elem in history:
                    self.assertTrue(date + timedelta(days=2) >= elem.date)
                    date = elem.date
                # recipients = list(self.backend.iter_transfer_recipients(id_or_account))
                # elf.assertTrue(len(recipients) > 0)
            elif account.type == Account.TYPE_MARKET:
                invest = list(self.backend.iter_investment(id_or_account))
                self.backend.iter_history(id_or_account)  # can be empty. Only try to call it
                self.assertTrue(len(invest) > 0)

    def test_subscriptions(self):
        l = list(self.backend.iter_subscription())
        for sub in l:
            _id = self.backend.get_subscription(sub.id)
            self.assertTrue(_id.id == sub.id)
            bills = list(self.backend.iter_documents(sub))
            self.assertTrue(len(bills) > 0)
            _id = self.backend.get_document(bills[0].id)
            self.assertTrue(_id.id == bills[0].id)
            self.backend.download_document(bills[0].id)



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.capabilities.messages import CapMessages, CapMessagesPost, Thread, Message
from weboob.tools.value import Value

from .browser import GuerrillamailBrowser


__all__ = ['GuerrillamailModule']


class GuerrillamailModule(Module, CapMessages, CapMessagesPost):
    NAME = 'guerrillamail'
    DESCRIPTION = u'GuerrillaMail temp mailbox'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = GuerrillamailBrowser

    CONFIG = BackendConfig(Value('inbox', label='Inbox', default=''))

    def iter_threads(self):
        inbox = self.config['inbox'].get()
        if not inbox:
            raise NotImplementedError()
        else:
            return [self.get_thread(inbox)]

    def get_thread(self, _id):
        t = Thread(_id)
        t.title = 'Mail for %s' % _id
        t.flags = t.IS_DISCUSSION

        first = True
        for d in self.browser.get_mails(_id):
            m = self.make_message(d, t)

            if not m.content:
                m.content = self.browser.get_mail_content(m.id)

            if first:
                first = False
                t.root = m
            else:
                m.parent = t.root
                m.parent.children.append(m)

        return t

    def post_message(self, m):
        raise NotImplementedError()
        for receiver in m.receivers:
            self.browser.send_mail(m.sender, receiver, m.title, m.content)

    def make_message(self, d, thread):
        m = Message(thread, d['id'])
        m.children = []
        m.sender = d['from']
        m.flags = 0
        if not d.get('read', True):
            m.flags = m.IS_UNREAD
        m.title = d['subject']
        m.date = d['datetime']
        m.receivers = [d['to']]
        return m



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.tools.backend import Module
from weboob.capabilities.calendar import CapCalendarEvent, CATEGORIES

from .browser import SenscritiqueBrowser
from .calendar import SensCritiquenCalendarEvent

__all__ = ['SenscritiqueModule']


class SenscritiqueModule(Module, CapCalendarEvent):
    NAME = 'senscritique'
    DESCRIPTION = u'senscritique website'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    ASSOCIATED_CATEGORIES = [CATEGORIES.TELE]
    BROWSER = SenscritiqueBrowser

    def search_events(self, query):
        if self.has_matching_categories(query):
            return self.list_events(query.start_date,
                                    query.end_date)

    def list_events(self, date_from, date_to=None):
        items = []
        for item in self.browser.list_events(date_from, date_to):
            items.append(item)

        items.sort(key=lambda o: o.start_date)
        return items

    def get_event(self, _id, event=None):
        return self.browser.get_event(_id, event)

    def fill_obj(self, event, fields):
        return self.get_event(event.id, event)

    OBJECTS = {SensCritiquenCalendarEvent: fill_obj}



# -*- coding: utf-8 -*-

# Copyright(C) 2012-2013 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from decimal import Decimal

from weboob.capabilities.bill import CapDocument, Detail, Subscription
from weboob.tools.application.repl import ReplApplication, defaultcount
from weboob.tools.application.formatters.iformatter import PrettyFormatter
from weboob.tools.application.base import MoreResultsAvailable
from weboob.core import CallErrors

__all__ = ['Boobill']


class SubscriptionsFormatter(PrettyFormatter):
    MANDATORY_FIELDS = ('id', 'label')

    def get_title(self, obj):
        if obj.renewdate:
            return u"%s - %s" % (obj.label, obj.renewdate.strftime('%d/%m/%y'))
        return obj.label


class Boobill(ReplApplication):
    APPNAME = 'boobill'
    VERSION = '1.3'
    COPYRIGHT = 'Copyright(C) 2012-YEAR Florent Fourcot'
    DESCRIPTION = 'Console application allowing to get/download documents and bills.'
    SHORT_DESCRIPTION = "get/download documents and bills"
    CAPS = CapDocument
    COLLECTION_OBJECTS = (Subscription, )
    EXTRA_FORMATTERS = {'subscriptions':   SubscriptionsFormatter,
                        }
    DEFAULT_FORMATTER = 'table'
    COMMANDS_FORMATTERS = {'subscriptions':   'subscriptions',
                           'ls':              'subscriptions',
                          }

    def main(self, argv):
        self.load_config()
        return ReplApplication.main(self, argv)

    def exec_method(self, id, method):
        l = []
        id, backend_name = self.parse_id(id)

        if not id:
            for subscrib in self.get_object_list('iter_subscription'):
                l.append((subscrib.id, subscrib.backend))
        else:
            l.append((id, backend_name))

        more_results = []
        not_implemented = []
        self.start_format()
        for id, backend in l:
            names = (backend,) if backend is not None else None
            try:
                for result in self.do(method, id, backends=names):
                    self.format(result)
            except CallErrors as errors:
                for backend, error, backtrace in errors:
                    if isinstance(error, MoreResultsAvailable):
                        more_results.append(id + u'@' + backend.name)
                    elif isinstance(error, NotImplementedError):
                        if backend not in not_implemented:
                            not_implemented.append(backend)
                    else:
                        self.bcall_error_handler(backend, error, backtrace)

        if len(more_results) > 0:
            print('Hint: There are more results available for %s (use option -n or count command)' % (', '.join(more_results)), file=self.stderr)
        for backend in not_implemented:
            print(u'Error(%s): This feature is not supported yet by this backend.' % backend.name, file=self.stderr)

    def do_subscriptions(self, line):
        """
        subscriptions

        List all subscriptions.
        """
        return self.do_ls(line)

    def do_details(self, id):
        """
        details [ID]

        Get details of subscriptions.
        If no ID given, display all details of all backends.
        """
        l = []
        id, backend_name = self.parse_id(id)

        if not id:
            for subscrib in self.get_object_list('iter_subscription'):
                l.append((subscrib.id, subscrib.backend))
        else:
            l.append((id, backend_name))

        for id, backend in l:
            names = (backend,) if backend is not None else None
            # XXX: should be generated by backend? -Flo
            # XXX: no, but you should do it in a specific formatter -romain
            # TODO: do it, and use exec_method here. Code is obsolete
            mysum = Detail()
            mysum.label = u"Sum"
            mysum.infos = u"Generated by boobill"
            mysum.price = Decimal("0.")

            self.start_format()
            for detail in self.do('get_details', id, backends=names):
                self.format(detail)
                mysum.price = detail.price + mysum.price

            self.format(mysum)

    def do_balance(self, id):
        """
        balance [ID]

        Get balance of subscriptions.
        If no ID given, display balance of all backends.
        """

        self.exec_method(id, 'get_balance')

    @defaultcount(10)
    def do_history(self, id):
        """
        history [ID]

        Get the history of subscriptions.
        If no ID given, display histories of all backends.
        """
        self.exec_method(id, 'iter_bills_history')

    @defaultcount(10)
    def do_documents(self, id):
        """
        documents [ID]

        Get the list of documents for subscriptions.
        If no ID given, display documents of all backends
        """
        self.exec_method(id, 'iter_documents')

    @defaultcount(10)
    def do_bills(self, id):
        """
        bills [ID]

        Get the list of bills documents for subscriptions.
        If no ID given, display bills of all backends
        """
        self.exec_method(id, 'iter_bills')

    def do_download(self, line, force_pdf=False):
        """
        download [ID | all] [FILENAME]

        download ID [FILENAME]

        download the document
        id is the identifier of the document (hint: try documents command)
        FILENAME is where to write the file. If FILENAME is '-',
        the file is written to stdout.

        download all [ID]

        You can use special word "all" and download all documents of
        subscription identified by ID.
        If Id not given, download documents of all subscriptions.
        """
        id, dest = self.parse_command_args(line, 2, 1)
        id, backend_name = self.parse_id(id)
        if not id:
            print('Error: please give a document ID (hint: use documents command)', file=self.stderr)
            return 2

        names = (backend_name,) if backend_name is not None else None

        # Special keywords, download all documents of all subscriptions
        if id == "all":
            callback = self.download_all if not force_pdf else self.download_all_pdf

            if dest is None:
                for subscription in self.do('iter_subscription', backends=names):
                    callback(subscription.id, names)
                return
            else:
                callback(dest, names)
                return

        if dest is None:
            for document in self.do('get_document', id, backends=names):
                dest = id + "." + (document.format if not force_pdf else 'pdf')

        if 'document' not in locals():
            document = id

        for buf in self.do('download_document' if not force_pdf else 'download_document_pdf', document, backends=names):
            if buf:
                if dest == "-":
                    print(buf)
                else:
                    try:
                        with open(dest, 'wb') as f:
                            f.write(buf)
                    except IOError as e:
                        print('Unable to write document in "%s": %s' % (dest, e), file=self.stderr)
                        return 1
                return

    def do_download_pdf(self, line):
        """
        download_pdf [id | all]

        download function with forced PDF conversion.
        """

        return self.do_download(line, force_pdf=True)

    def download_all(self, id, names, force_pdf=False):
        id, backend_name = self.parse_id(id)

        for document in self.do('iter_documents', id, backends=names):
            dest = document.id + "." + (document.format if not force_pdf else 'pdf')

            for buf in self.do('download_document' if not force_pdf else 'download_document_pdf', document, backends=names):
                if buf:
                    if dest == "-":
                        print(buf)
                    else:
                        try:
                            with open(dest, 'wb') as f:
                                f.write(buf)
                        except IOError as e:
                            print('Unable to write bill in "%s": %s' % (dest, e), file=self.stderr)
                            return 1
        return

    def download_all_pdf(self, id, names):
        """
        download_pdf all

        download_all function with forced PDF conversion.
        """

        return self.download_all(id, names, force_pdf=True)



# -*- coding: utf-8 -*-

# Copyright(C) 2012      Gabriel Serme
# Copyright(C) 2011      Gabriel Kerneis
# Copyright(C) 2010-2011 Jocelyn Jaubert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.bank import CapBankTransferAddRecipient, Account, AccountNotFound
from weboob.capabilities.profile import CapProfile
from weboob.capabilities.contact import CapContact
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword, ValueBool, Value

from .browser import BoursoramaBrowser


__all__ = ['BoursoramaModule']


class BoursoramaModule(Module, CapBankTransferAddRecipient, CapProfile, CapContact):
    NAME = 'boursorama'
    MAINTAINER = u'Gabriel Kerneis'
    EMAIL = 'gabriel@kerneis.info'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = u'Boursorama'
    CONFIG = BackendConfig(ValueBackendPassword('login',      label='Identifiant', masked=False),
                           ValueBackendPassword('password',   label='Mot de passe'),
                           ValueBool('enable_twofactors',     label='Send validation sms', default=False),
                           Value('device',                    label='Device name', regexp='\w*', default=''),
                           Value('pin_code',                  label='Sms code', required=False),
                          )
    BROWSER = BoursoramaBrowser

    def create_default_browser(self):
        return self.create_browser(self.config)

    def iter_accounts(self):
        return self.browser.get_accounts_list()

    def get_account(self, _id):
        account = self.browser.get_account(_id)
        if account:
            return account
        else:
            raise AccountNotFound()

    def iter_history(self, account):
        for tr in self.browser.get_history(account):
            if not tr._is_coming:
                yield tr

    def iter_coming(self, account):
        for tr in self.browser.get_history(account, coming=True):
            if tr._is_coming:
                yield tr

    def iter_investment(self, account):
        return self.browser.get_investment(account)

    def get_profile(self):
        return self.browser.get_profile()

    def iter_contacts(self):
        return self.browser.get_advisor()

    def iter_transfer_recipients(self, account):
        if not isinstance(account, Account):
            account = self.get_account(account)
        return self.browser.iter_transfer_recipients(account)

    def init_transfer(self, transfer, **kwargs):
        return self.browser.init_transfer(transfer, **kwargs)

    def new_recipient(self, recipient, **kwargs):
        return self.browser.new_recipient(recipient, **kwargs)

    def execute_transfer(self, transfer, **kwargs):
        return self.browser.execute_transfer(transfer, **kwargs)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.messages import CapMessages, Message, Thread
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value

from .browser import FourChan


__all__ = ['FourChanModule']


class FourChanModule(Module, CapMessages):
    NAME = 'fourchan'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = '4chan image board'
    CONFIG = BackendConfig(Value('boards', label='Boards to fetch'))
    STORAGE = {'boards': {}}
    BROWSER = FourChan

    def _splitid(self, id):
        return id.split('.', 1)

    def get_thread(self, id):
        thread = None

        if isinstance(id, Thread):
            thread = id
            id = thread.id

        if '.' not in id:
            self.logger.warning('Malformated ID (%s)' % id)
            return

        board, thread_id = self._splitid(id)

        with self.browser:
            _thread = self.browser.get_thread(board, thread_id)

        flags = 0
        if _thread.id not in self.storage.get('boards', board, default={}):
            flags |= Message.IS_UNREAD

        if not thread:
            thread = Thread(id)
        thread.title = _thread.filename
        thread.root = Message(thread=thread,
                              id=0,  # root message
                              title=_thread.filename,
                              sender=_thread.author,
                              receivers=None,
                              date=_thread.datetime,
                              parent=None,
                              content=_thread.text,
                              signature=None,
                              children=[],
                              flags=flags|Message.IS_HTML)

        for comment in _thread.comments:
            flags = 0
            if comment.id not in self.storage.get('boards', board, _thread.id, default=[]):
                flags |= Message.IS_UNREAD

            m = Message(thread=thread,
                        id=comment.id,
                        title=_thread.filename,
                        sender=comment.author,
                        receivers=None,
                        date=comment.datetime,
                        parent=thread.root,
                        content=comment.text,
                        signature=None,
                        children=None,
                        flags=flags|Message.IS_HTML)
            thread.root.children.append(m)

        return thread

    def iter_threads(self):
        for board in self.config['boards'].get().split(' '):
            with self.browser:
                threads = self.browser.get_threads(board)
            for thread in threads:
                t = Thread('%s.%s' % (board, thread.id))
                t.title = thread.filename
                yield t

    def iter_unread_messages(self):
        for thread in self.iter_threads():
            self.fill_thread(thread, 'root')

            for m in thread.iter_all_messages():
                if m.flags & Message.IS_UNREAD:
                    yield m

    def set_message_read(self, message):
        board, thread_id = self._splitid(message.thread.id)
        self.storage.set('boards', board, thread_id, self.storage.get('boards', board, thread_id, default=[]) + [message.id])
        self.storage.save()

    def fill_thread(self, thread, fields):
        return self.get_thread(thread)

    OBJECTS = {Thread: fill_thread}



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from base64 import b64decode

from weboob.capabilities.paste import CapPaste, BasePaste
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.capabilities.paste import bin_to_b64
from weboob.tools.value import Value

from .browser import JirafeauBrowser


__all__ = ['JirafeauModule']


class JirafeauPaste(BasePaste):
    @property
    def page_url(self):
        return self.url


class JirafeauModule(Module, CapPaste):
    NAME = 'jirafeau'
    DESCRIPTION = u'Jirafeau-based file upload website'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    CONFIG = BackendConfig(Value('base_url', label='Base Jirafeau URL',
                                 description='URL of the Jirafeau-based site to use',
                                 regexp=r'https?://.*',
                                 default='http://jirafeau.net/'))

    BROWSER = JirafeauBrowser

    def create_default_browser(self):
        return self.create_browser(self.config['base_url'].get())

    def can_post(self, contents, title=None, public=None, max_age=None):
        if public or max_age not in self.browser.age_keyword:
            return 0

        max_size, _ = self.browser.get_max_sizes()
        if max_size and len(b64decode(contents)) > max_size:
            return 0
        return 1

    def get_paste(self, url):
        d = self.browser.recognize(url)
        if not d:
            return
        if self.browser.exists(d['id']):
            return

        ret = JirafeauPaste(d['id'])
        ret.url = d['url']
        return ret

    def new_paste(self, *args, **kwargs):
        return JirafeauPaste(*args, **kwargs)

    def post_paste(self, paste, max_age=None):
        d = self.browser.post(b64decode(paste.contents), paste.title, max_age)
        paste.id = d['id']
        paste.url = d['page_url']
        return paste

    def fill_paste(self, obj, fields):
        if 'contents' in fields:
            data = self.browser.download(obj.id)
            obj.contents = bin_to_b64(data)

    OBJECTS = {JirafeauPaste: fill_paste}



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.tools.test import BackendTest

from random import choice


class PodnapisiTest(BackendTest):
    MODULE = 'podnapisi'

    def test_subtitle(self):
        lsub = []
        subtitles = self.backend.iter_subtitles('fr', 'spiderman')
        for i in range(5):
            subtitle = subtitles.next()
            lsub.append(subtitle)
        assert (len(lsub) > 0)

        # get the file of a random sub
        if len(lsub):
            subtitle = choice(lsub)
            self.backend.get_subtitle_file(subtitle.id)
            ss = self.backend.get_subtitle(subtitle.id)
            assert ss.url.startswith('http')



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Simon Lipp
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import LoginBrowser, URL, need_login
from weboob.browser.pages import RawPage, JsonPage, HTMLPage
from weboob.browser.exceptions import ClientError
from weboob.exceptions import BrowserIncorrectPassword

class LoginPage(HTMLPage):
    def login(self, username, password):
        try:
            self.browser.login_result.open(data = {
                "email": username,
                "password": password,
                "remember": "false"
            })
        except ClientError as e:
            if e.response.status_code == 403:
                raise BrowserIncorrectPassword()
            else:
                raise

        self.logged = True

class OpenEDXBrowser(LoginBrowser):
    login = URL('/login', LoginPage)
    login_result = URL("/user_api/v1/account/login_session/", RawPage)
    threads = URL(r'/courses/(?P<course>.+)/discussion/forum/\?ajax=1&page=(?P<page>\d+)&sort_key=date&sort_order=desc', JsonPage)
    messages = URL(r'/courses/(?P<course>.+)/discussion/forum/(?P<topic>.+)/threads/(?P<id>.+)\?ajax=1&resp_skip=(?P<skip>\d+)&resp_limit=100', JsonPage)
    thread = URL(r'/courses/(?P<course>.+)/discussion/forum/(?P<topic>.+)/threads/(?P<id>.+)', HTMLPage)

    def __init__(self, url, course, *args, **kwargs):
        self.BASEURL = url
        self.course = course
        LoginBrowser.__init__(self, *args, **kwargs)

    def prepare_request(self, req):
        token = self.session.cookies.get("csrftoken")
        if token:
            req.headers.setdefault("X-CSRFToken", token)
        if self.threads.match(req.url) or self.messages.match(req.url):
            req.headers.setdefault("X-Requested-With", "XMLHttpRequest")
        return LoginBrowser.prepare_request(self, req)

    def do_login(self):
        self.login.stay_or_go()
        self.page.login(self.username, self.password)

    @need_login
    def get_threads(self, page=1):
        return self.threads.open(course = self.course, page = page)

    @need_login
    def get_thread(self, topic, id, skip):
        return self.messages.open(course = self.course,
                topic = topic, id = id, skip = skip)



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import datetime
import re
from dateutil.relativedelta import relativedelta

from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.pages import HTMLPage, LoggedPage, PartialHTMLPage
from weboob.browser.filters.standard import CleanText, CleanDecimal, Currency as CleanCurrency
from weboob.browser.filters.html import AbsoluteLink
from weboob.capabilities.bank import Account
from weboob.capabilities import NotAvailable
from weboob.exceptions import ActionNeeded
from weboob.tools.capabilities.bank.transactions import FrenchTransaction as Transaction
from weboob.tools.compat import urljoin
from weboob.tools.date import ChaoticDateGuesser, parse_french_date


def parse_decimal(s):
    # we don't know which decimal format this account will use
    comma = s.rfind(',') > s.rfind('.')
    return CleanDecimal(replace_dots=comma).filter(s)


class WrongLoginPage(HTMLPage):
    pass


class AccountSuspendedPage(HTMLPage):
    pass


class LoginPage(HTMLPage):
    def login(self, username, password):
        form = self.get_form(name='ssoform')
        form['UserID'] = username
        form['USERID'] = username
        form['Password'] = password
        form['PWD'] = password
        form.submit()


class AccountsPage(LoggedPage, PartialHTMLPage):
    def get_account(self):
        for div in self.doc.xpath('.//div[@id="card-details"]'):
            a = Account()
            a.id = CleanText().filter(div.xpath('.//span[@class="acc-num"]'))
            a.label = CleanText().filter(div.xpath('.//span[@class="card-desc"]'))
            if "carte" in a.label.lower():
                a.type = Account.TYPE_CARD
            balance = CleanText().filter(div.xpath('.//span[@class="balance-data"]'))
            if balance in (u'Indisponible', u'Indisponible Facturation en cours', ''):
                a.balance = NotAvailable
            else:
                a.currency = a.get_currency(balance)
                a.balance = - abs(parse_decimal(balance))

            # Cancel card don't have a link to watch history
            link = self.doc.xpath('.//div[@class="wide-bar"]/h3/a')
            if len(link) == 1:
                a.url = urljoin(self.url, link[0].attrib['href'])
            else:
                a.url = None

            return a

    def get_idx_list(self):
        fetched = False
        for div in self.doc.xpath('//div[@id="card-list"]//div[has-class("card-details")]'):
            _id = div.attrib['id']
            idx = re.match(r'card-(\d+)-detail', _id).group(1)

            message = CleanText('.//div[has-class("messages")]')(div).lower()
            cancelled = ('annul' in message or 'cancel' in message)

            yield idx, cancelled
            fetched = True

        if fetched:
            return

        for div in self.doc.xpath('//div[@id="card-detail"]'):
            idx = div.xpath('//span[@id="cardSortedIndex"]/@data')[0]
            message = CleanText('.//div[has-class("messages")]')(div).lower()
            cancelled = ('annul' in message or 'cancel' in message)

            yield idx, cancelled
            return

    def get_session(self):
        return self.doc.xpath('//form[@id="j-session-form"]//input[@name="Hidden"]/@value')


class AccountsPage2(LoggedPage, PartialHTMLPage):
    @method
    class iter_accounts(ListElement):
        item_xpath = '//table[@id="summaryTable"]'

        class item(ItemElement):
            klass = Account

            def condition(self):
                if 'Votre carte est annulée' in CleanText('.//span[@id="cardSORStatus"]')(self):
                    self.logger.warning('skipping cancelled card %r', self.obj_id(self))
                    return False
                return True

            obj_id = CleanText('.//td[@class="cardArtColWidth"]/div[@class="summaryTitles"]')
            obj_label = CleanText('.//span[@class="cardTitle"]')
            obj_type = Account.TYPE_CARD

            obj_currency = CleanCurrency('.//td[@id="colOSBalance"]/div[@class="summaryValues makeBold"]')

            def obj_balance(self):
                return -abs(parse_decimal(CleanText('.//td[@id="colOSBalance"]/div[@class="summaryValues makeBold"]')(self)))

            obj_url = AbsoluteLink('.//a[text()="View Latest Transactions"]', default=AbsoluteLink('.//a[span[text()="Online Statement"] or text()="Détail de vos opérations"]'))


class TransactionsPage(LoggedPage, HTMLPage):
    def is_last(self):
        current = False
        for option in self.doc.xpath('//select[@id="viewPeriod"]/option'):
            if 'selected' in option.attrib:
                current = True
            elif current:
                return False

        return True

    def get_end_debit_date(self):
        for option in self.doc.xpath('//select[@id="viewPeriod"]/option'):
            if 'selected' in option.attrib:
                m = re.search('(\d+) ([\w\.]+) (\d{4})$', option.text.strip(), re.UNICODE)
                if m:
                    return datetime.date(int(m.group(3)),
                                         self.parse_month(m.group(2)),
                                         int(m.group(1)))

    def get_beginning_debit_date(self):
        for option in self.doc.xpath('//select[@id="viewPeriod"]/option'):
            if 'selected' in option.attrib:
                m = re.search('^(\d+) ([\w\.]+) (\d{4})', option.text.strip(), re.UNICODE)
                if m:
                    return datetime.date(int(m.group(3)),
                                         self.parse_month(m.group(2)),
                                         int(m.group(1)))
        return datetime.date.today()

    COL_DATE = 0
    COL_TEXT = 1
    COL_CREDIT = -2
    COL_DEBIT = -1

    FR_MONTHS = ['janv', u'févr', u'mars', u'avr', u'mai', u'juin', u'juil', u'août', u'sept', u'oct', u'nov', u'déc']
    US_MONTHS = ['Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec']

    @classmethod
    def parse_month(cls, s):
        # there can be fr or us labels even if currency is EUR
        s = s.rstrip('.')
        try:
            return cls.FR_MONTHS.index(s) + 1
        except ValueError:
            return cls.US_MONTHS.index(s) + 1

    def get_history(self, currency):
        # checking if the card is still valid
        if self.doc.xpath('//div[@id="errorbox"]'):
            return

        # adding a time delta because amex have hard time to put the date in a good interval
        beginning_date = self.get_beginning_debit_date() - datetime.timedelta(days=360)
        end_date = self.get_end_debit_date()

        guesser = ChaoticDateGuesser(beginning_date, end_date)

        # Since the site doesn't provide the debit_date,
        # we just use the date of beginning of the previous period.
        # If this date + 1 month is greater than today's date,
        # then the transaction is coming
        end_of_period = None
        previous_date = CleanText('//td[@id="colStatementBalance"]/div[3]', default=None)(self.doc)
        if previous_date:
            end_of_period = (parse_french_date(' '.join(previous_date.split()[1:4])) + relativedelta(months=1)).date()
        else:
            previous_date = CleanText('//select[@id="viewPeriod"]/option[@selected]', default=None)(self.doc)
            if previous_date:
                end_of_period = parse_french_date(' '.join(previous_date.split()[:3])) + relativedelta(days=-1) + relativedelta(months=1)
                end_of_period = end_of_period.date()

        for tr in reversed(self.doc.xpath('//div[@id="txnsSection"]//tbody/tr[@class="tableStandardText"]')):
            cols = tr.findall('td')

            t = Transaction()

            day, month = CleanText().filter(cols[self.COL_DATE]).split(' ', 1)
            day = int(day)
            month = self.parse_month(month)
            date = guesser.guess_date(day, month)

            vdate = None
            try:
                detail = cols[self.COL_TEXT].xpath('./div[has-class("hiddenROC")]')[0]
            except IndexError:
                pass
            else:
                m = re.search(r' (\d{2} \D{3,4})', (' '.join([txt.strip() for txt in detail.itertext()])).strip())
                if m:
                    vday, vmonth = m.group(1).strip().split(' ')
                    vday = int(vday)
                    vmonth = self.parse_month(vmonth)
                    vdate = guesser.guess_date(vday, vmonth)
                detail.drop_tree()

            raw = (' '.join([txt.strip() for txt in cols[self.COL_TEXT].itertext()])).strip()
            credit = CleanText().filter(cols[self.COL_CREDIT])
            debit = CleanText().filter(cols[self.COL_DEBIT])
            if end_of_period is not None and datetime.date.today() < end_of_period:
                t._is_coming = True
            else:
                t._is_coming = False

            t.date = t.rdate = date
            t.vdate = vdate
            t.raw = re.sub(r'[ ]+', ' ', raw)
            t.label = re.sub('(.*?)( \d+)?  .*', r'\1', raw).strip()
            t.amount = parse_decimal(credit or debit) * (1 if credit else -1)
            if t.amount > 0:
                t.type = t.TYPE_ORDER
            else:
                t.date = end_of_period
                t.type = t.TYPE_DEFERRED_CARD

            yield t


class ActionNeededPage(LoggedPage, HTMLPage):
    def on_load(self):
        if self.doc.xpath('//meta[contains(@content,"action/home?request_type=un_Activation")]'):
            raise ActionNeeded()



<%inherit file="layout.py"/>
from weboob.tools.capabilities.gallery.genericcomicreadertest import GenericComicReaderTest


class ${r.classname}BackendTest(GenericComicReaderTest):
    MODULE = '${r.name}'

    def test_download(self):
        return self._test_download('${r.download_id}')



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re
try:
    from html.parser import HTMLParser
except ImportError:
    from HTMLParser import HTMLParser

from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.elements import method, ListElement, ItemElement, SkipItem
from weboob.capabilities.collection import Collection
from weboob.browser.filters.standard import CleanText


class PageLogin(HTMLPage):
    def login(self, email, password, csrf):
        form = self.get_form(xpath='//form[contains(@class,"login-form")]')
        form['email'] = email
        form['password'] = password
        form['csrfmiddlewaretoken'] = csrf
        form.submit()


class PageDashboard(LoggedPage, HTMLPage):
    def iter_courses(self):
        for c in self.doc.xpath('//article[@class="course"]'):
            title = c.xpath('.//h3[@class="course-title"]/a')[0].text.strip()
            link = c.xpath('.//a[contains(@class,"enter-course")]')[0]
            url = self.browser.absurl(link.get('href'))
            match = self.browser.course.match(url)
            courseid = match.group('course').replace('/', '-')
            yield Collection([courseid], title)


class PageChapter(LoggedPage, HTMLPage):
    @method
    class iter_chapters(ListElement):
        item_xpath = '//a[has-class("button-chapter")]'

        class item(ItemElement):
            klass = Collection

            obj_title = CleanText('.')

            def obj_split_path(self):
                # parse first section link
                section_links = self.xpath('./following-sibling::div[has-class("chapter-content-container")]//a')
                if not section_links:
                    raise SkipItem()
                url = section_links[0].get('href')
                url = self.page.browser.absurl(url)
                match = self.page.browser.section.match(url)
                courseid = self.env['course'].replace('/', '-')
                chapter = match.group('chapter')
                return [courseid, chapter]

            def obj_id(self):
                return '-'.join(self.obj_split_path())

    @method
    class iter_sections(ListElement):
        item_xpath = '//div[has-class("chapter-content-container")]//div[has-class("menu-item")]//a'

        class item(ItemElement):
            klass = Collection

            obj_title = CleanText('.')

            def obj_split_path(self):
                url = self.xpath('.')[0].get('href')
                url = self.page.browser.absurl(url)
                match = self.page.browser.section.match(url)
                courseid = self.env['course'].replace('/', '-')
                chapter = match.group('chapter')
                section = match.group('section')
                return [courseid, chapter, section]

            def obj_id(self):
                return '-'.join(self.obj_split_path())


def unescape(s):
    return HTMLParser().unescape(s)


class PageSection(LoggedPage, HTMLPage):
    video_url = re.compile(r'[^\s;]+/HD\.mp4', re.I)
    video_thumb = re.compile(r'reposter=&#34;(.*?)&#34;')
    video_title = re.compile(r'&lt;h2&gt;(.*?)&lt;/h2&gt;')

    def iter_videos(self):
        urls = set()

        # this part of the site contains escaped HTML...
        for n, page_match in enumerate(self.video_url.finditer(self.text)):
            url = page_match.group(0)
            match = self.browser.file.match(url)

            if url in urls:
                # prevent duplicate urls
                continue
            urls.add(url)

            beforetext = self.text[:page_match.end(0)]
            try:
                thumb = list(self.video_thumb.finditer(beforetext))[-1].group(1)
            except IndexError:
                thumb = None
            try:
                title = unescape(unescape(list(self.video_title.finditer(beforetext))[-1].group(1)))
            except IndexError:
                title = u'%s - %s' % (match.group('id'), n)

            yield {
                'url': url,
                'title': title,
                'thumbnail': thumb,
            }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import subprocess
import sys

if '--deps' in sys.argv:
    sys.argv.remove('--deps')
    deps = []
else:
    deps = ['--nodeps']

print("Weboob local installer")
print()
if len(sys.argv) < 2:
    print("This tool will install Weboob to be usuable without requiring")
    print("messing with your system, which should only be touched by a package manager.")
    print()
    print("Usage: %s DESTINATION [OPTIONS]" % sys.argv[0])
    print()
    print("By default, no dependencies are installed, as you should try")
    print("to install them from your package manager as much as possible.")
    print("To install all the missing dependencies, add the option --deps")
    print("at the end of the command line.")
    print()
    print("Error: Please provide a destination, "
          "for example ‘%s/bin’" % os.getenv('HOME'), file=sys.stderr)
    sys.exit(1)
else:
    dest = os.path.expanduser(sys.argv[1])

print("Installing weboob applications into ‘%s’." % dest)

subprocess.check_call(
    [sys.executable, 'setup.py',
        'install', '--user', '--install-scripts=%s' % dest] + sys.argv[2:] + deps,
    cwd=os.path.join(os.path.dirname(__file__), os.pardir))

subprocess.check_call([sys.executable, os.path.join(dest, 'weboob-config'), 'update'])

print()
print("Installation done. Applications are available in ‘%s’." % dest)
print("You can remove the source files.")
print()
print("To have easy access to the Weboob applications,")
print("you should add the following line to your ~/.bashrc or ~/.zshrc file:")
print("export PATH=\"$PATH:%s\"" % dest)
print("And then restart your shells.")



# -*- coding: utf-8 -*-
# Copyright(C) 2010-2015 Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.capabilities.messages import CantSendMessage

from weboob.capabilities.bill import Bill, Subscription
from weboob.browser.pages import HTMLPage, JsonPage, LoggedPage
from weboob.browser.filters.json import Dict
from weboob.browser.filters.standard import CleanDecimal, CleanText, Env, Format, Regexp, Field
from weboob.browser.filters.html import Link
from weboob.browser.elements import DictElement, ItemElement, ListElement, method
from weboob.tools.date import parse_french_date

class LoginPage(HTMLPage):
    def login(self, login, password):
        form = self.get_form('//form[@id="log_data"]')

        form['username'] = login
        form['password'] = password

        form.submit()


class HomePage(HTMLPage, LoggedPage):
    pass


class ProfilePage(JsonPage, LoggedPage):
    @method
    class get_list(DictElement):
        item_xpath = 'data/lignes'

        class item(ItemElement):
            klass = Subscription

            obj__type = CleanText(Dict('type'))
            obj_label = Env('label')
            obj_subscriber = Format("%s %s %s", CleanText(Dict('civilite')),
                                    CleanText(Dict('prenom')), CleanText(Dict('nom')))

            def obj_id(self):
                if Dict('date-activation')(self) is not None:
                    return Format('%s-%s', Dict('num_ligne'), Dict('date-activation'))(self)
                else:
                    return Format('%s', Dict('num_ligne'))(self)

            def parse(self, el):
                # add spaces
                number = iter(Field('id')(self).split('-')[0])
                self.env['label'] = ' '.join(a+b for a, b in zip(number, number))


class SendSMSPage(HTMLPage):
    def send_sms(self, message, receivers):
        sms_number = CleanDecimal(Regexp(CleanText('//span[@class="txt12-o"][1]/strong'), '(\d*) SMS.*'))(self.doc)

        if sms_number == 0:
            msg = CleanText('//span[@class="txt12-o"][1]')(self.doc)
            raise CantSendMessage(msg)

        form = self.get_form('//form[@name="formSMS"]')
        form["fieldMsisdn"] = receivers
        form["fieldMessage"] = message.content

        form.submit()


class SendSMSErrorPage(HTMLPage):
    def get_error_message(self):
        return CleanText('//span[@class="txt12-o"][1]')(self.doc)


class DocumentsPage(HTMLPage):
    def get_ref(self, label):
        options = self.doc.xpath('//select[@id="factureMois"]/option[position() > 1]/@value')

        for option in options:
            for ctr in self.doc.xpath('//div[has-class("eccoetape")]'):
                if ctr.xpath('.//span[contains(text(), $label)]', label=label):
                    ref = ctr.xpath('.//a[@id="btnAnciennesFactures"]')
                    if ref:
                        return re.search('reference=([\d]+)', Link().filter(ref)).group(1)

            self.logger.debug("couldn't find ref this month, retrying with %s", option)
            self.doc = self.browser.open('%s?mois=%s' % (self.browser.url, option)).page.doc
        return None

    @method
    class get_documents(ListElement):
        item_xpath = '//div[@facture-id]'

        class item(ItemElement):
            klass = Bill

            obj__ref = CleanText('//input[@id="noref"]/@value')
            obj_id = Format('%s_%s', Env('subid'), CleanText('./@facture-id'))
            obj_url = Format('http://www.bouyguestelecom.fr/parcours/facture/download/index?id=%s&no_reference=%s', CleanText('./@facture-id'), CleanText('./@facture-ligne'))
            obj_date = Env('date')
            obj_format = u"pdf"
            obj_label = CleanText('./text()')
            obj_type = u"bill"
            obj_price = CleanDecimal(CleanText('./span', replace=[(u' € ', '.')]))
            obj_currency = u"€"

            def parse(self, el):
                self.env['date'] = parse_french_date('01 %s' % CleanText('./text()')(self)).date()

            def condition(self):
                # XXX ugly fix to avoid duplicate bills
                return CleanText('./@facture-id')(self.el) != CleanText('./following-sibling::div[1]/@facture-id')(self.el)


class UselessPage(HTMLPage):
    pass
